/*---------------------------------------------------------------------------*
Project: OLV
File: olv_Api.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_API_H_
#define __OLV_API_H_

/*! @file
@brief These are API declarations for the OLV library.
*/

#include <nn/olv/olv_Types.h>
#include <nn/olv/olv_Result.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

/*@defgroup function Static Functions
 *  @{
 */

/*!
@brief Initializes the OLV library.

You must call this function before calling any of the functions in the <tt>nn::olv</tt> namespace. <br />
Note that the working buffer specified for the <span class="argument">initializeParam</span> argument with the <tt>@ref nn::olv::InitializeParam::SetWork</tt> function must not be cleared until the <tt>@ref nn::olv::Finalize</tt> function is called. <br />

- OLV Access Key Settings<br />
To use the OLV library at the development stage, configure and build the application with the unique ID issued by Nintendo, and then set the OLV access key issued by Nintendo using the CTR Config tool before starting the application.
This prevents people who are not involved in application development from posting to the viewable communities.

@param[out] initializedData         The class that stores the initialization results.
@param[in]  initializeParam         The class that stores the parameters used for initialization.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result Initialize(nn::olv::InitializedData* initializedData, const nn::olv::InitializeParam* initializeParam);

/*!
@brief Finalizes the OLV library.
*/
void Finalize();

/*!
@brief Processes OLV library connections.

This function causes Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />

@param[out] connectedData          The class that stores the connection results.
@param[in]  connectParam            The class that stores the parameters used for the connection.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result Connect(nn::olv::ConnectedData* connectedData, const nn::olv::ConnectParam* connectParam);

/*!
@brief Cancels the Internet communication process.

This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result Cancel();

/*!
@brief Determines whether the OLV library has been initialized.

This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@return Returns <tt>true</tt> if the library is initialized, and <tt>false</tt> otherwise.
*/
bool IsInitialized();

/*!
@brief Determines whether the OLV library has been connected to.

This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@return Returns <tt>true</tt> if the library has been connected to, and <tt>false</tt> otherwise.
*/
bool IsConnected();

/*!
@brief Gets the error code.

This function is thread safe.

@param[in] result   Specifies the result obtained from the OLV library API.

@return Returns the error code corresponding to the specified result. <br />
Returns <tt>0</tt> if the specified result is not from the OLV library API.
@retval     0                                   There are no OLV-related corresponding error codes.
@retval     Anything else                    The error code.
*/
u32 GetErrorCode(const nn::olv::Result& result);

/*!
@brief Starts the Miiverse application.

If this function succeeds, you must call the <tt>nn::applet::WaitForStarting</tt> function to wait for the start event of the application. <br />
Note that the working buffer specified for the <span class="argument">startParam</span> argument with the <tt>@ref nn::olv::StartPortalAppParam::SetWork</tt> function must not be cleared until either the <tt>@ref nn::olv::Resume</tt> or the <tt>nn::olv::Finalize</tt> function is called after the start event of the application.

@param[in] startParam   The class that stores the parameters used when starting the Miiverse application.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result StartPortalApp(const nn::olv::StartPortalAppParam* startParam);

/*!
@brief Handles recovery operations.

Call this function when the start event of the application comes after the OLV library has been initialized or the <tt>@ref nn::olv::StartPortalApp</tt> function has been called. <br />
If system parameters are being passed from the Miiverse application, that data can be received.

@param[out] resumedData         The class that stores the recovery process results.
@param[in] resumeParam          The class that stores the parameters used for the recovery process.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result Resume(nn::olv::ResumedData* resumedData, const nn::olv::ResumeParam* resumeParam);

/*!
@brief Downloads post data.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] topicData               The class that stores the downloaded topic tags.
@param[out] postDataList            An array of classes that stores the downloaded post data. <br />
Specify the number of arrays in the <span class="argument">downloadParam</span> argument of the <tt>@ref nn::olv::DownloadPostDataListParam::SetPostDataMaxNum</tt> function.
@param[out] downloadedPostDataNum   The buffer that stores the number of downloaded posts.
@param[in] downloadParam            The class that stores the parameters used when downloading posts.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result DownloadPostDataList(nn::olv::DownloadedTopicData* topicData, nn::olv::DownloadedPostData* postDataList, u32* downloadedPostDataNum, const nn::olv::DownloadPostDataListParam* downloadParam);

/*!
@brief Downloads comment data.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] commentDataList            An array of classes that store the downloaded comment data. <br />
Specify the number of arrays using the <tt>@ref nn::olv::DownloadCommentDataListParam::SetCommentDataMaxNum</tt> function in the <span class="argument">downloadParam</span> parameter.
@param[out] downloadedCommentDataNum  The buffer that stores the amount of downloaded comment data.
@param[in] downloadParam            The class that stores the parameters used when downloading comments.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result DownloadCommentDataList(nn::olv::DownloadedCommentData* commentDataList, u32* downloadedCommentDataNum, const nn::olv::DownloadCommentDataListParam* downloadParam);


/*!
@brief Downloads community data.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] communityDataList           An array of classes that store the downloaded community data. <br />
Specify the number of arrays in the <span class="argument">downloadParam</span> argument of the <tt>@ref nn::olv::DownloadCommunityDataListParam::SetCommunityDataMaxNum</tt> function.
@param[out] downloadedCommunityDataNum  The buffer that stores the number of downloaded communities.
@param[in] downloadParam                The class that stores the parameters used when downloading community data.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result DownloadCommunityDataList(nn::olv::DownloadedCommunityData* communityDataList, u32* downloadedCommunityDataNum, const nn::olv::DownloadCommunityDataListParam* downloadParam);

/*!
@brief Downloads user data.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] userDataList             An array of classes that store the downloaded user data. <br />
Specify the number of arrays using the <tt>@ref nn::olv::DownloadUserDataListParam::SetUserDataMaxNum</tt> function in the <span class="argument">downloadParam</span> parameter.
@param[out] downloadedUserDataNum  The buffer that stores the amount of downloaded user data.
@param[in] downloadParam            The class that stores the parameters used when downloading user data.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result DownloadUserDataList(nn::olv::DownloadedUserData* userDataList, u32* downloadedUserDataNum, const nn::olv::DownloadUserDataListParam* downloadParam);

/*!
@brief Creates, updates, or deletes community data.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] uploadedData            The class that stores the results of creating or updating community data. <br />
You can specify <tt>NULL</tt> if not needed. <br />
Nothing is stored if the community data is deleted.
@param[in] uploadParam              The class that stores the parameters used when sending community data.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result UploadCommunityData(nn::olv::UploadedCommunityData* uploadedData, const nn::olv::UploadCommunityDataParam* uploadParam);

/*!
@brief Gives a Yeah to a post or removes a Yeah from a post.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[in] uploadParam              The class that stores the parameters used when giving a Yeah to a post or removing a Yeah from a post.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result UploadEmpathyToPostData(const nn::olv::UploadEmpathyToPostDataParam* uploadParam);

/*!
@brief Registers a community as a favorite or removes a community as a favorite.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[in] uploadParam              The class that stores the parameters used when registering a community as a favorite or removing a community as a favorite.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result UploadFavoriteToCommunityData(const nn::olv::UploadFavoriteToCommunityDataParam* uploadParam);

/*!
@brief Uploads whether to follow a user or to stop following a user.

This function can only be used when a connection to the OLV library has already been made with the <tt>@ref nn::olv::Connect</tt> function. <br />
This function initiates Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[in] uploadParam              The class that stores the parameters when uploading to follow a user or to stop following a user.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
*/
nn::olv::Result UploadFollowToUserData(const nn::olv::UploadFollowToUserDataParam* uploadParam);


/** @} */

}   // namespace olv
}   // namespace nn
#endif
