/*---------------------------------------------------------------------------*
Project: OLV
File: olv_Result.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef NN_OLV_OLV_RESULT_H_
#define NN_OLV_OLV_RESULT_H_

/*! @file
@brief These are result declarations for the OLV library.
*/

#include <nn/Result.h>

#ifdef __cplusplus

namespace nn {
namespace olv {

/// @cond
class Result : public nn::Result
{
private:
    static const bit32 MASK_FAIL_BIT        = 0x80000000u; // Most significant bit.

    static const s32 SIZE_DESCRIPTION       = 20;
    static const s32 SIZE_MODULE            =  9;
    static const s32 SIZE_LEVEL             =  3;

    static const s32 SHIFTS_DESCRIPTION     = 0;
    static const s32 SHIFTS_MODULE          = SHIFTS_DESCRIPTION + SIZE_DESCRIPTION;
    static const s32 SHIFTS_LEVEL           = SHIFTS_MODULE      + SIZE_MODULE;
    //NN_STATIC_ASSERT( SHIFTS_LEVEL + SIZE_LEVEL == 32 );

    static const u32 MASK_NEGATIVE_LEVEL  = (~0u) << SIZE_LEVEL;

#define NN_RESULT_H_MAKE_MASK(size, shift)  (((~0u) >> (32 - (size))) << (shift))
#define NN_RESULT_H_MAKE_MASK_HELPER(c)     \
    static const u32 MASK_ ## c           = NN_RESULT_H_MAKE_MASK(SIZE_ ## c, SHIFTS_ ## c)
    NN_RESULT_H_MAKE_MASK_HELPER(DESCRIPTION);
    NN_RESULT_H_MAKE_MASK_HELPER(MODULE);
    NN_RESULT_H_MAKE_MASK_HELPER(LEVEL);
#undef NN_RESULT_H_MAKE_MASK_HELPER
#undef NN_RESULT_H_MAKE_MASK

#define NN_RESULT_H_MAKE_MAX(size)          ((~0u) >> (32 - (size)))
#define NN_RESULT_H_MAKE_MAX_HELPER(c)      \
    static const s32 MAX_ ## c              = NN_RESULT_H_MAKE_MAX(SIZE_ ## c)
    NN_RESULT_H_MAKE_MAX_HELPER(DESCRIPTION);
    NN_RESULT_H_MAKE_MAX_HELPER(MODULE);
    NN_RESULT_H_MAKE_MAX_HELPER(LEVEL);
#undef NN_RESULT_H_MAKE_MAX_HELPER
#undef NN_RESULT_H_MAKE_MAX

public:
    /*
* @brief Enumerated type indicating the severity of an error. (Do not use this type for error handling.)
     */
    enum LevelImpl
    {
        LEVEL_SUCCESS    =  0,
        LEVEL_FATAL      = -1,
        LEVEL_USAGE      = -2,
        LEVEL_STATUS     = -3,

        LEVEL_END = -7
    };
    typedef int Level;

    /*
* @brief Enumerated type that indicates the details of errors. (Do not use this type for error handling.)
     */
    enum
    {
        DESCRIPTION_SUCCESS                         =    0,  // Succeeded.
        DESCRIPTION_INVALID_RESULT_VALUE            = MAX_DESCRIPTION -  0   // These values are not used.
    };

private:

    template <nn::Result::Level TLevel, int TSummary, int TModule, int TDescription, int TDescriptionEx> friend struct ConstRangeBase;

public:
    Result()
        : nn::Result(),
          m_CodeEx( static_cast<u32>(
                        ((static_cast<u32>(MAX_LEVEL)       << SHIFTS_LEVEL)         & MASK_LEVEL)        |
                        ((static_cast<u32>(MAX_MODULE)      << SHIFTS_MODULE)        & MASK_MODULE)       |
                        ((static_cast<u32>(MAX_DESCRIPTION) << SHIFTS_DESCRIPTION)   & MASK_DESCRIPTION) ))
    {
    }
    Result(nn::Result::Level level, nn::Result::Summary summary, nn::Result::Module module, int description, int descriptionEx)
        : nn::Result(level, summary, module, description),
          m_CodeEx( static_cast<u32>(
                        ((static_cast<u32>(ConvertLevel(level))        << SHIFTS_LEVEL)         & MASK_LEVEL)        |
                        ((static_cast<u32>(module)       << SHIFTS_MODULE)        & MASK_MODULE)       |
                        ((static_cast<u32>(descriptionEx)  << SHIFTS_DESCRIPTION)   & MASK_DESCRIPTION) ))
    {
    }

    /*
* @brief Returns a <tt>Description</tt> enumerated type indicating the details of the error.
     *
* Enumerated type definitions will change in the future. Do not use them for any purpose other than checking the content of errors during development.
     */
    int GetDescription() const
    {
        return static_cast<int>( GetCodeBitsEx(MASK_DESCRIPTION, SHIFTS_DESCRIPTION) );
    }

    /*!
* @brief Converts a <tt>Result</tt> code into a 32-bit array.
     *
* Do not use them for any purpose other than checking the content of errors during development.
     */
    u32 GetPrintableBits() const { return m_CodeEx; }

    /*
* @brief Checks for equality.
     *
* For information about handling errors, see the OLV library reference.
     *
* @param[in] rhs  Specifies the right-hand operand.
* @return Returns <tt>true</tt> if the operands are equal, and <tt>false</tt> otherwise.
     */
    bool operator ==(const Result& rhs) const { return this->m_CodeEx == rhs.m_CodeEx; }

    /*
* @brief Checks for inequality.
     *
* For information about handling errors, see the OLV library reference.
     *
* @param[in] rhs  Specifies the right-hand operand.
* @return Returns <tt>true</tt> if the operands are not equal, and <tt>false</tt> otherwise.
     */
    bool operator !=(const Result& rhs) const { return this->m_CodeEx != rhs.m_CodeEx; }

    template <nn::Result::Level TLevel, nn::Result::Summary TSummary, Result::Module TModule, int TDescription, int TDescriptionEx> struct ConstRangeBase;
    template <nn::Result::Level TLevel, nn::Result::Summary TSummary, nn::Result::Module TModule, int TDescription, int TDescriptionMin, int TDescriptionMax, int TDescriptionEx, int TDescriptionExStart, int TDescriptionExEnd> struct ConstRange;

private:
    nn::olv::Result::Level ConvertLevel(nn::Result::Level level) const
    {
        switch (level)
        {
        case nn::Result::LEVEL_SUCCESS:
            return nn::olv::Result::LEVEL_SUCCESS;
        case nn::Result::LEVEL_FATAL:
            return nn::olv::Result::LEVEL_FATAL;
        case nn::Result::LEVEL_USAGE:
            return nn::olv::Result::LEVEL_USAGE;
        default:
            return nn::olv::Result::LEVEL_STATUS;
        }
    }

    u32 GetCodeBitsEx(u32 mask, s32 shift) const
    {
        return ((m_CodeEx & mask) >> shift);
    }

private:
    u32 m_CodeEx;
};

struct ResultSuccess
{
    operator Result() const { return Result((nn::Result::Level)0, (nn::Result::Summary)0, (nn::Result::Module)0, 0, 0); }
};

template <nn::Result::Level TLevel, nn::Result::Summary TSummary, nn::Result::Module TModule, int TDescription, int TDescriptionEx>
struct Result::ConstRangeBase
{
    static const nn::Result::Level Level = TLevel;
    static const nn::Result::Summary Summary = TSummary;
    static const nn::Result::Module Module = TModule;
    static const int Description = TDescription;
    static const int DescriptionEx = TDescriptionEx;

    operator Result() const { return Result(Level, Summary, Module, Description, DescriptionEx); }
};

template <nn::Result::Level TLevel, nn::Result::Summary TSummary, nn::Result::Module TModule, int TDescription, int TDescriptionMin, int TDescriptionMax, int TDescriptionEx, int TDescriptionExStart, int TDescriptionExEnd>
struct Result::ConstRange : public Result::ConstRangeBase<TLevel, TSummary, TModule, TDescription, TDescriptionEx>
{
    static const int DescriptionMin = TDescriptionMin;
    static const int DescriptionMax = TDescriptionMax;
    static const int DescriptionExStart = TDescriptionExStart;
    static const int DescriptionExEnd = TDescriptionExEnd;

    static bool Includes(nn::Result result)
    {
        return result.GetModule() == TModule && TDescriptionMin <= result.GetDescription() && result.GetDescription() < TDescriptionMax;
    }
    static bool IncludesEx(Result result)
    {
        return result.GetModule() == TModule && (TDescriptionExStart <= result.GetDescription() && result.GetDescription() < TDescriptionExEnd);
    }

    friend bool operator<=(nn::Result lhs, const ConstRange&) { return ConstRange::Includes(lhs); }
    friend bool operator>=(const ConstRange&, nn::Result rhs) { return ConstRange::Includes(rhs); }
    friend bool operator<=(Result lhs, const ConstRange&) { return ConstRange::IncludesEx(lhs); }
    friend bool operator>=(const ConstRange&, Result rhs) { return ConstRange::IncludesEx(rhs); }
};

#define NN_OLV_DEFINE_RESULT_CONST_RANGE(name, level, summary, module, errorCode, errorCodeMin, errorCodeMax) \
    typedef ::nn::olv::Result::ConstRange<(level), (summary), (module), (errorCode / 100), (errorCodeMin / 100), (errorCodeMax / 100), (errorCode * 128), (errorCodeMin * 128), (errorCodeMax * 128)> name
/// @endcond

/** @addtogroup nn_olv_result Return Values
 *  @{
 */

/*!
@class    nn::olv::ResultFatalError
@brief     Fatal error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultFatalError, 
    nn::Result::LEVEL_FATAL, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 100, 100, 200
);

/*!
@class    nn::olv::ResultUsageError
@brief     Usage error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultUsageError, 
    nn::Result::LEVEL_USAGE, nn::Result::SUMMARY_INVALID_ARGUMENT, nn::Result::MODULE_NN_OLV, 200, 200, 300
);

/*!
@class    nn::olv::ResultCanceled
@brief     Canceled.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultCanceled, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_CANCELLED, nn::Result::MODULE_NN_OLV, 300, 300, 400
);

/*!
@class    nn::olv::ResultLibraryError
@brief     Library error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultLibraryError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 1000, 1000, 2000
);

/*!
@class    nn::olv::ResultLibraryPermanentError
@brief      Permanent library error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultLibraryPermanentError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 1000, 1000, 1100
);

/*!
@class    nn::olv::ResultLibraryTemporaryError
@brief      Temporary library error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultLibraryTemporaryError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 1100, 1100, 2000
);

/*!
@class    nn::olv::ResultNetworkError
@brief     Network error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultNetworkError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 2000, 2000, 5000
);

/*!
@class    nn::olv::ResultNetworkDataError
@brief     Network data error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultNetworkDataError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_WRONG_ARGUMENT, nn::Result::MODULE_NN_OLV, 2000, 2000, 3000
);

/*!
@class    nn::olv::ResultNetworkLibraryError
@brief     Network library error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultNetworkLibraryError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 3000, 3000, 4000
);

/*!
@class    nn::olv::ResultNetworkHttpError
@brief     HTTP error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultNetworkHttpError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 4000, 4000, 5000
);

/*!
@class    nn::olv::ResultServerError
@brief     Server error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultServerError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 5000, 5000, 6000
);

/*!
@class    nn::olv::ResultServerPermanentError
@brief      Permanent server error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultServerPermanentError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 5000, 5000, 5100
);

/*!
@class    nn::olv::ResultServerTemporaryError
@brief      Temporary server error.
*/
NN_OLV_DEFINE_RESULT_CONST_RANGE(
    ResultServerTemporaryError, 
    nn::Result::LEVEL_STATUS, nn::Result::SUMMARY_INVALID_STATE, nn::Result::MODULE_NN_OLV, 5100, 5100, 6000
);

/** @} */

} // namespace olv
} // namespace nn


#endif // __cplusplus

#endif // NN_OLV_OLV_RESULT_H_
