/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesDownloadCommentData.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_DOWNLOAD_COMMENTDATA_H_
#define __OLV_TYPES_DOWNLOAD_COMMENTDATA_H_

/*! @file
@brief      Declaration of types related to downloading OLV comment data.
*/

#include <nn/olv/olv_TypesDownloadedDataBase.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief Represents the parameters when downloading comment data.
*/
class DownloadCommentDataListParam
{
public:
    /*!
@brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.

- You can specify either <tt>@ref FLAG_FILTER_BY_BODY_TEXT</tt> or <tt>@ref FLAG_FILTER_BY_BODY_MEMO</tt> to filter the comments to download.
- Currently <tt>@ref FLAG_FILTER_BY_SELF</tt>, <tt>@ref FLAG_FILTER_BY_FRIEND</tt>, and <tt>@ref FLAG_FILTER_BY_FOLLOW</tt> are ignored if specified. (Support is planned for future versions.)
- Specify <tt>@ref FLAG_WITH_SPOILER</tt> to download comments that include spoilers.
- If you specify a comment ID using the <tt>@ref SetCommentId</tt> function, the only flag that can be specified is <tt>@ref FLAG_WITH_MII_DATA</tt>.
    */
    enum
    {
        FLAG_NONE = 0,                          //!< This is the state when no flags are specified.
        FLAG_FILTER_BY_FRIEND       = (1<<0),   //!< Download comments from users who are friends.
        FLAG_FILTER_BY_FOLLOW       = (1<<1),   //!< Download comments from users you are following.
        FLAG_FILTER_BY_SELF         = (1<<2),   //!< Download your own comments.
        FLAG_FILTER_BY_BODY_TEXT    = (1<<3),   //!< Download only text comments.
        FLAG_FILTER_BY_BODY_MEMO    = (1<<4),   //!< Download only handwritten memo comments.
        FLAG_DISTINCT_PID           = (1<<5),   //!< Get only a single comment from each user.
        FLAG_WITH_MII_DATA          = (1<<6),   //!< Include Mii character data in the downloaded comment data.
        FLAG_WITH_SPOILER           = (1<<8)    //!< Also download comments that include spoilers.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadCommentDataListParam();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

Specifying this option is not required.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the post ID for the comment data to download. <br />

You must specify this option.

@param[in]  postId      The post ID. <br />
    */
    void SetPostId(const char* postId);

    /*!
@brief Sets the comment ID. <br />

Specifying this option is not required. <br />
If you specify a comment ID, only the comment with the set ID is downloaded. <br />

@param[in]  commentId      The comment ID. <br />
    */
    void SetCommentId(const char* commentId);

    /*!
@brief Sets the date the comment was made.

Specifying this option is not required. <br />
If you specify a date, only comments that were made on the specified date are downloaded. <br />
Set a date that adjusts for the difference from the time zone of the account.

@param[in]  postDate        The date the comment was made.
    */
    void SetPostDate(const nn::fnd::DateTime postDate);

    /*!
@brief Sets the language ID.

Specifying this option is not required. If no language ID is set, comments in the language set in System Settings are downloaded. <br />
See <tt>nn::cfg::CfgLanguageCode</tt> for the language IDs. <br />
Specify <tt>@ref nn::olv::LANGUAGE_ID_ALL</tt> to download comments in all languages. <br />

@param[in]  languageId      The language ID.
    */
    void SetLanguageId(u8 languageId);

    /*!
@brief  Sets the maximum length of text data.

Specifying this option is not required. If not specified, the maximum length defaults to 100 characters. <br />
Text data that exceeds the maximum character length is truncated at the maximum number of characters and downloaded.

@param[in]  bodyTextMaxLength   The maximum character length for text data. <br />
The range of values that can be specified is from <tt>1</tt> to <tt>@ref nn::olv::BODY_TEXT_MAX_LENGTH</tt>.
    */
    void SetBodyTextMaxLength(u32 bodyTextMaxLength);

    /*!
@brief Sets the maximum number of comments to download.

You must specify this option.

@param[in]  commentDataMaxNum  The maximum number of comments to download. <br />
Make sure that you specify a value equal to or greater than this maximum value for the number of arrays in the <span class="argument">commentDataList</span> argument to the <tt>@ref nn::olv::DownloadCommentDataList</tt> function. <br />

    */
    void SetCommentDataMaxNum(u32 commentDataMaxNum);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommentDataList</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
This function returns fixed values.

@return Returns the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommentDataList</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommentDataList</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>GetWorkSize</tt> function. <br />

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u8  m_LanguageId;
    s8  m_IsSetLanguageId;
    s8  m_IsSetBodyTextMaxLength;
    NN_PADDING1;
    nn::fnd::DateTime m_PostDate;
    u32 m_BodyTextMaxLength;
    u32 m_CommentDataMaxNum;
    char  m_PostId[nn::olv::POST_ID_BUFF_LENGTH];
    char  m_CommentId[nn::olv::COMMENT_ID_BUFF_LENGTH];
    u8 m_Reserved[1024-96];

    friend class nn::olv::internal::Main;
};

/*!
@brief A class that stores the downloaded comment data.
*/
class DownloadedCommentData : public DownloadedDataBase
{
public:
    /*!
@brief Flags that can be specified in the <tt>TestFlags</tt> function.
    */
    enum
    {
        FLAG_NONE = 0,                                                                      //!< This is the state when no flags are specified.
        FLAG_WITH_BODY_TEXT             = DOWNLOAD_FLAG_VALUE_WITH_BODY_TEXT,               //!< A flag indicating that the comment is a text comment.
        FLAG_WITH_BODY_MEMO             = DOWNLOAD_FLAG_VALUE_WITH_BODY_MEMO,               //!< A flag indicating that the comment is a handwritten memo comment.
        FLAG_WITH_EXTERNAL_IMAGE_DATA   = DOWNLOAD_FLAG_VALUE_WITH_EXTERNAL_IMAGE_DATA,     //!< A flag indicating that the comment includes an attached image.
        FLAG_WITH_MII_DATA              = DOWNLOAD_FLAG_VALUE_WITH_MII_DATA,                //!< A flag indicating that the comment includes attached Mii character data.
        FLAG_WITH_APP_DATA              = DOWNLOAD_FLAG_VALUE_WITH_APP_DATA,                //!< A flag indicating that the comment includes application data.
        FLAG_FREE_FORMAT                = DOWNLOAD_FLAG_VALUE_FREE_FORMAT,                  //!< A flag indicating that the comment is a free text comment.
        FLAG_SPOILER                    = DOWNLOAD_FLAG_VALUE_SPOILER                       //!< A flag indicating that the comment includes spoilers.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadedCommentData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Tests flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).

@return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise.
    */
    bool TestFlags(u32 flags) const;

    /*!
@brief Gets the principal ID of the user.

@return Returns the principal ID of the user.
    */
    u32 GetUserPid() const;

    /*!
@brief Gets the date the comment was made.

The date returned reflects the difference in the time zone of the account.

@return Returns the date the comment was made.
    */
    nn::fnd::DateTime GetPostDate() const;

    /*!
@brief      Gets the mood.

@return Returns the mood.
    */
    s8 GetFeeling() const;

    /*!
@brief Gets the region ID.

@return Returns the region ID.
    */
    u32 GetRegionId() const;

    /*!
@brief Gets the platform ID.

@return Returns the platform ID.
    */
    u8 GetPlatformId() const;

    /*!
@brief Gets the language ID.

See <tt>nn::cfg::CfgLanguageCode</tt> for the language IDs.

@return Returns the language ID.
    */
    u8 GetLanguageId() const;

    /*!
@brief Gets the country ID.

See <tt>nn::cfg::CfgCountryCode</tt> for the country IDs.

@return Returns the country ID.
    */
    u8 GetCountryId() const;

    /*!
@brief Gets the Mii data of the user.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_MII_DATA</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_MII_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">storeData</span> argument.

@param[out] storeData       The buffer that stores the Mii data. <br />
    */
    void GetMiiData(nn::mii::StoreData* storeData) const;

    /*!
@brief Gets the nickname of the Mii character of the user.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">miiNickname</span> argument.
- If a value less than that of <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt> is specified for the <span class="argument">miiNicknameMaxLength</span> argument.

@param[out] miiNickname             The buffer that stores the nickname of the Mii character. <br />
@param[in]  miiNicknameMaxLength    The length of the buffer that stores the nickname of the Mii character, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt>.
    */
    void GetMiiNickname(wchar_t* miiNickname, u32 miiNicknameMaxLength) const;

    /*!
@brief Gets text data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_BODY_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_BODY_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">bodyText</span> argument.
- If a value less than that of <tt>@ref nn::olv::BODY_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">bodyTextMaxLength</span> argument.

@param[out] bodyText            The buffer that stores the text data.
@param[in]  bodyTextMaxLength   The length of the buffer that stores the text data, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::BODY_TEXT_BUFF_LENGTH</tt>.
    */
    void GetBodyText(wchar_t* bodyText, u32 bodyTextMaxLength) const;

    /*!
@brief Gets the handwritten memo data.

This function can only be used when the <tt>@ref TestFlags</tt> function is set to <tt>@ref FLAG_WITH_BODY_MEMO</tt> and returns <tt>true</tt>. 
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- When the <tt>@ref TestFlags</tt> function is set to <tt>@ref FLAG_WITH_BODY_MEMO</tt> and returns <tt>false</tt>.
- When the <span class="argument">bodyMemo</span> parameter is set to <tt>NULL</tt>.
- When a value less than <tt>@ref nn::olv::BODY_MEMO_BUFF_LENGTH</tt> is specified for the <span class="argument">bodyMemoMaxSize</span> parameter.

@param[out] bodyMemo            The buffer that stores the handwritten memo data.
@param[out] bodyMemoSize        The buffer that stores the size of the handwritten memo data.
@param[in]  bodyMemoMaxSize     The size of the buffer that stores the handwritten memo data. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::BODY_MEMO_BUFF_LENGTH</tt>.
    */
    void GetBodyMemo(u8* bodyMemo, u32* bodyMemoSize, u32 bodyMemoMaxSize) const;

    /*!
@brief Gets application data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_APP_DATA</tt> is specified.
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">appData</span> argument.
- If a value less than that returned by the <tt>@ref GetAppDataSize</tt> function is specified for the <span class="argument">appDataMaxSize</span> argument.

@param[out] appData             The buffer that stores the application data.
@param[out] appDataSize         The buffer that stores the size of the application data.
@param[in]  appDataMaxSize      The size of the buffer that stores the application data. <br />
Make sure that you specify a value of at least that returned by the <tt>@ref GetAppDataSize</tt> function.
    */
    void GetAppData(u8* appData, u32* appDataSize, u32 appDataMaxSize) const;

    /*!
@brief Gets the size of the application data.

@return Returns the size of the application data.
    */
    u32 GetAppDataSize() const;

    /*!
@brief Gets the size of the attached image data.

@return Returns the size of the attached image data.
    */
    u32 GetExternalImageDataSize() const;

    /*!
@brief Downloads the attached image data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_EXTERNAL_IMAGE_DATA</tt> is specified. <br />
This function causes Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] externalImageData            The buffer that stores the attached image data.
@param[out] externalImageDataSize        The buffer that stores the size of the attached image data.
@param[in]  externalImageDataMaxSize     The size of the buffer that stores the attached image data.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
    */
    nn::olv::Result DownloadExternalImageData(void* externalImageData, u32* externalImageDataSize, u32 externalImageDataMaxSize) const;

    /*!
@brief      Gets the comment ID.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">commentId</span> argument.
- If a value less than that of <tt>@ref nn::olv::COMMENT_ID_BUFF_LENGTH</tt> is specified for the <span class="argument">commentIdMaxLength</span> argument.

@param[out] commentId              The buffer that stores the comment ID.
@param[in]  commentIdMaxLength     The length of the buffer that stores the comment ID, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::COMMENT_ID_BUFF_LENGTH</tt>.
    */
    void GetCommentId(char* commentId, u32 commentIdMaxLength) const;

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
