/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesDownloadCommunityData.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_DOWNLOAD_COMMUNITY_H_
#define __OLV_TYPES_DOWNLOAD_COMMUNITY_H_

/*! @file
@brief      Declaration of types related to downloading OLV community data.
*/

#include <nn/olv/olv_TypesDownloadedDataBase.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief Represents the parameters when downloading community data.
*/
class DownloadCommunityDataListParam
{
public:
    /*!
@brief Flags that can be specified in the <tt>SetFlags</tt> function.

- If not using the <tt>@ref SetCommunityId</tt> function, make sure that you specify one of the following: <tt>@ref FLAG_FILTER_BY_FAVORITE</tt>, <tt>@ref FLAG_FILTER_BY_OFFICIAL</tt>, or <tt>@ref FLAG_FILTER_BY_SELF</tt>. <br />
- If the <tt>@ref SetCommunityId</tt> function is used to specify a community ID, you cannot specify <tt>FLAG_FILTER_BY_*</tt>.
    */
    enum Flags
    {
        FLAG_NONE = 0,                      //!< This is the state when no flags are specified.
        FLAG_FILTER_BY_FAVORITE = (1<<0),   //!< Downloads the community data for communities registered as favorites (communities that you have created are treated as favorites and are downloaded).
        FLAG_FILTER_BY_OFFICIAL = (1<<1),   //!< Downloads the community data for official communities.
        FLAG_FILTER_BY_SELF     = (1<<2),   //!< Downloads the community data for communities that you have created.
        FLAG_WITH_MII_DATA      = (1<<3),   //!< Includes the Mii character data for the user who created the community in the community data to download.
        FLAG_WITH_ICON_DATA     = (1<<4)    //!< Includes icon data in the community data to download.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadCommunityDataListParam();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

You must specify this option when the <tt>@ref SetCommunityId</tt> function is not used.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the maximum number of communities to download.

You must specify this option.

@param[in]  communityDataMaxNum     The maximum number of communities to download. <br />
Make sure that you specify a value of <tt>1</tt> or greater. <br />
If you have specified community IDs with the <tt>@ref SetCommunityId</tt> function, make sure that you specify the number of communities beyond the ones specified to download. <br />
Make sure that you specify a value equal to or greater than this maximum value for the number of arrays in the <span class="argument">communityDataList</span> argument to the <tt>@ref nn::olv::DownloadCommunityDataList</tt> function. <br />
    */
    void SetCommunityDataMaxNum(u32 communityDataMaxNum);

    /*!
@brief Sets the community ID.

Specifying this option is not required. <br />
If you specify a community ID, only the community with the specified ID is downloaded. <br />
Up to a maximum of <tt>@ref nn::olv::COMMUNITY_ID_MAX_NUM</tt> community IDs can be specified.

@param[in]  communityId     The community ID.
@param[in]  index       The index. <br />
The range of values that can be specified is from <tt>0</tt> to <tt>@ref nn::olv::COMMUNITY_ID_MAX_NUM - 1</tt>.
    */
    void SetCommunityId(u32 communityId, u8 index);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommunityDataList</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
This function returns fixed values.

@return Returns the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommunityDataList</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the size of the working buffer used by the <tt>@ref nn::olv::DownloadCommunityDataList</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>@ref GetWorkSize</tt> function. <br />

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_CommunityDataMaxNum;
    u32 m_CommunityIds[COMMUNITY_ID_MAX_NUM];
    s8 m_IsSetCommunityIds[COMMUNITY_ID_MAX_NUM];
    NN_PADDING2;
    u8 m_Reserved[256-168];

    friend class nn::olv::internal::Main;
};

/*!
@brief A class that stores the downloaded community data.
*/
class DownloadedCommunityData
{
public:
    /*!
@brief Flags that can be specified in the <tt>TestFlags</tt> function.
    */
    enum Flags
    {
        FLAG_NONE = 0,                                                              //!< This is the state when no flags are specified.
        FLAG_WITH_TITLE_TEXT        = DOWNLOAD_FLAG_VALUE_WITH_TITLE_TEXT,          //!< A flag indicating that the community name is included in the community data.
        FLAG_WITH_DESCRIPTION_TEXT  = DOWNLOAD_FLAG_VALUE_WITH_DESCRIPTION_TEXT,    //!< A flag indicating that the community description is included in the community data.
        FLAG_WITH_APP_DATA          = DOWNLOAD_FLAG_VALUE_WITH_APP_DATA,            //!< A flag indicating that application data is included in the community data.
        FLAG_WITH_ICON_DATA         = DOWNLOAD_FLAG_VALUE_WITH_ICON_DATA,           //!< A flag indicating that icon data is included in the community data.
        FLAG_WITH_MII_DATA          = DOWNLOAD_FLAG_VALUE_WITH_MII_DATA             //!< A flag indicating that the Mii character data for the user who created the community is included in the community data.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadedCommunityData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Tests flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).

@return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise.
    */
    bool TestFlags(u32 flags) const;

    /*!
@brief Gets the community ID.

@return Returns the community ID.
    */
    u32 GetCommunityId() const;

    /*!
@brief Gets the community code.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">communityCode</span> argument.
- If a value less than that of <tt>@ref nn::olv::COMMUNITY_CODE_BUFF_LENGTH</tt> is specified for the <span class="argument">communityCodeMaxLength</span> argument.

@param[out] communityCode               The buffer for storing the community code.
@param[in]  communityCodeMaxLength      The length of the buffer that stores the community code, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::COMMUNITY_CODE_BUFF_LENGTH</tt>.
    */
    void GetCommunityCode(char* communityCode, u32 communityCodeMaxLength) const;

    /*!
@brief Gets the principal ID of the user who created the community.

@return Returns the principal ID of the user who created the community.
    */
    u32 GetOwnerPid() const;

    /*!
@brief Gets the community name.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_TITLE_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_TITLE_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">titleText</span> argument.
- If a value less than that of <tt>@ref nn::olv::TITLE_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">titleTextMaxLength</span> argument.

@param[out] titleText           The buffer for storing the community name.
@param[in]  titleTextMaxLength  The length of the buffer that stores the community name, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::TITLE_TEXT_BUFF_LENGTH</tt>.
    */
    void GetTitleText(wchar_t* titleText, u32 titleTextMaxLength) const;

    /*!
@brief Gets the community description.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_DESCRIPTION_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_DESCRIPTION_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument>descriptionText</span> argument.
- If a value less than that of <tt>@ref nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">descriptionTextMaxLength</span> argument.

@param[out] descriptionText             The buffer for storing the community description.
@param[in]  descriptionTextMaxLength    The length of the buffer that stores the community description, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH</tt>.
    */
    void GetDescriptionText(wchar_t* descriptionText, u32 descriptionTextMaxLength) const;

    /*!
@brief Gets application data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_APP_DATA</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">appData</span> argument.
- If a value less than that returned by the <tt>@ref GetAppData</tt> function is specified for the <span class="argument">appDataMaxSize</span> argument.

@param[out] appData             The buffer that stores the application data.
@param[out] appDataSize         The buffer that stores the size of the application data.
@param[in]  appDataMaxSize      The size of the buffer that stores the application data. <br />
Make sure that you specify a value of at least that returned by the <tt>@ref GetAppData</tt> function.
    */
    void GetAppData(u8* appData, u32* appDataSize, u32 appDataMaxSize) const;

    /*!
@brief Gets the size of the application data.

@return Returns the size of the application data.
    */
    u32 GetAppDataSize(void) const;

    /*!
@brief Gets icon data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_ICON_DATA</tt> is specified.

@param[out] iconData            The buffer that stores the icon data.
@param[out] iconDataSize        The buffer that stores the size of the icon data.
@param[in]  iconDataMaxSize     The size of the buffer that stores the icon data. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::ICON_DATA_BUFF_LENGTH</tt>.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
    */
    nn::olv::Result GetIconData(u8* iconData, u32* iconDataSize, u32 iconDataMaxSize) const;

    /*!
@brief Gets the Mii character data of the user who created the community.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_MII_DATA</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">storeData</span> argument.

@param[out] storeData       The buffer that stores the Mii data of the user who created the community. <br />
    */
    void GetOwnerMiiData(nn::mii::StoreData* storeData) const;

    /*!
@brief Gets the nickname of the Mii character of the user who created the community.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">miiNickname</span> argument.
- If a value less than that of <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt> is specified for the <span class="argument">miiNicknameMaxLength</span> argument.

@param[out] miiNickname             The buffer that stores the nickname of the Mii character. <br />
@param[in]  miiNicknameMaxLength    The length of the buffer that stores the nickname of the Mii character, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt>.
    */
    void GetOwnerMiiNickname(wchar_t* miiNickname, u32 miiNicknameMaxLength) const;

private:
    u32 m_Flags;
    u32 m_CommunityId;
    u32 m_OwnerPid;
    wchar_t m_TitleText[TITLE_TEXT_BUFF_LENGTH];
    u32 m_TitleTextLength;
    wchar_t m_DescriptionText[DESCRIPTION_TEXT_BUFF_LENGTH];
    u32 m_DescriptionTextLength;
    u8  m_AppData[APP_DATA_BUFF_LENGTH];
    u32 m_AppDataSize;
    u8  m_IconData[COMPRESSED_ICON_DATA_BUFF_LENGTH];
    s32 m_IconDataSize;
    u8  m_MiiData[MII_DATA_BUFF_LENGTH];
    wchar_t m_MiiNickname[MII_NICKNAME_BUFF_LENGTH];
    u8 m_Reserved[12288-11240];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
