/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesDownloadPostData.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_DOWNLOAD_POSTDATA_H_
#define __OLV_TYPES_DOWNLOAD_POSTDATA_H_

/*! @file
@brief      Declaration of types related to downloading OLV post data.
*/

#include <nn/olv/olv_TypesDownloadedDataBase.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief Represents the parameters when downloading post data.
*/
class DownloadPostDataListParam
{
public:
    /*!
@brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.

- You can specify <tt>@ref FLAG_FILTER_BY_FRIEND</tt> / <tt>@ref FLAG_FILTER_BY_FOLLOW</tt> / <tt>@ref FLAG_FILTER_BY_SELF</tt> in combinations to filter the posts to download. <br />
These flags cannot be used while using the <tt>SetSearchPid</tt> function and specifying a principal ID. <br />
- You can specify either of <tt>@ref FLAG_FILTER_BY_BODY_TEXT</tt> or <tt>@ref FLAG_FILTER_BY_BODY_MEMO</tt> to filter the posts to download. <br />
- Specify <tt>@ref FLAG_WITH_SPOILER</tt> to download posts that include spoilers.
    */
    enum
    {
        FLAG_NONE = 0,                          //!< This is the state when no flags are specified.
        FLAG_FILTER_BY_FRIEND       = (1<<0),   //!< Download posts from users who are friends.
        FLAG_FILTER_BY_FOLLOW       = (1<<1),   //!< Download posts from users you are following.
        FLAG_FILTER_BY_SELF         = (1<<2),   //!< Download your own posts.
        FLAG_FILTER_BY_BODY_TEXT    = (1<<3),   //!< Download only text posts.
        FLAG_FILTER_BY_BODY_MEMO    = (1<<4),   //!< Download only handwritten memo posts.
        FLAG_DISTINCT_PID           = (1<<5),   //!< Get only a single post from each user.
        FLAG_WITH_MII_DATA          = (1<<6),   //!< Include Mii character data in the post data to download.
        FLAG_WITH_EMPATHY_ADDED     = (1<<7),   //!< Include flags for whether you have already given Yeahs to posts in the post data to download.
        FLAG_WITH_SPOILER           = (1<<8)    //!< Also download posts that include spoilers.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadPostDataListParam();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

Specifying this option is not required.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the language ID.

Specifying this option is not required. If no language ID is specified, posts in the language set in the System Settings are downloaded. <br />
See <tt>nn::cfg::CfgLanguageCode</tt> for the language IDs. <br />
Specify <tt>@ref nn::olv::LANGUAGE_ID_ALL</tt> to download posts in all languages. <br />

@param[in]  languageId      The language ID.
    */
    void SetLanguageId(u8 languageId);

    /*!
@brief Sets the community ID.

Specifying this option is not required. If no community ID is specified, posts from the general community are downloaded.

@param[in]  communityId     The community ID.
    */
    void SetCommunityId(u32 communityId);

    /*!
@brief Sets the search key.

Specifying this option is not required. <br />
If you specify a search key, only posts with the specified search key are downloaded.

@param[in]  searchKey       The search key. <br />
Use UTF-16LE for the encoding. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::SEARCH_KEY_MAX_LENGTH</tt>.
    */
    void SetSearchKey(const wchar_t* searchKey);

    /*!
@brief Sets the principal ID of the user.

Specifying this option is not required. <br />
If you specify a principal ID, only the posts from the user with the specified principal ID are downloaded.

@param[in]  searchPid      The principal ID of the user.
    */
    void SetSearchPid(u32 searchPid);

    /*!
@brief Sets the post ID.

Specifying this option is not required. <br />
If you specify a post ID, only the post with the specified ID is downloaded. <br />
Up to a maximum of <tt>@ref nn::olv::POST_ID_MAX_NUM</tt> post IDs can be specified.

@param[in]  postId      The post ID. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::POST_ID_BUFF_LENGTH - 1</tt>.
@param[in]  index       The index. <br />
The range of values that can be specified is from <tt>0</tt> to <tt>@ref nn::olv::POST_ID_MAX_NUM - 1</tt>.
    */
    void SetPostId(const char* postId, u32 index);

    /*!
@brief Sets the date the post was made.

Specifying this option is not required. <br />
If you specify a date, only posts that were made prior to the specified date are downloaded. <br />
Specify a date that adjusts for the difference from the time zone of the account.

@param[in]  postDate        The date the post was made.
    */
    void SetPostDate(const nn::fnd::DateTime postDate);

    /*!
@brief  Sets the maximum length of text data.

Specifying this option is not required. If not specified, the maximum length defaults to 100 characters. <br />
Text data that exceeds the maximum character length is truncated at the maximum number of characters and downloaded.

@param[in]  bodyTextMaxLength   The maximum character length for text data. <br />
The range of values that can be specified is from <tt>1</tt> to <tt>@ref nn::olv::BODY_TEXT_MAX_LENGTH</tt>.
    */
    void SetBodyTextMaxLength(u32 bodyTextMaxLength);

    /*!
@brief Sets the maximum number of posts to download.

You must specify this option.

@param[in]  postDataMaxNum  The maximum number of posts to download. <br />
If you specify post IDs with the <tt>@ref SetPostId</tt> function, make sure that you specify the number of posts greater than the number of the post IDs specified to download. <br />
Make sure that you specify a value equal to or greater than this maximum value for the number of arrays in the <span class="argument">postDataList</span> argument to the <tt>nn::olv::DownloadPostDataList</tt> function. <br />

    */
    void SetPostDataMaxNum(u32 postDataMaxNum);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::DownloadPostDataList</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
This function returns fixed values.

@return Returns the size of the working buffer used by the <tt>@ref nn::olv::DownloadPostDataList</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the size of the working buffer used by the <tt>@ref nn::olv::DownloadPostDataList</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>@ref GetWorkSize</tt> function. <br />

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_BodyTextMaxLength;
    u32 m_CommunityId;
    u32 m_SearchPid;
    u8  m_LanguageId;
    s8  m_IsSetLanguageId;
    s8  m_IsSetCommunityId;
    s8  m_IsSetBodyTextMaxLength;
    u32 m_PostDataMaxNum;
    wchar_t m_SearchKey[SEARCH_KEY_BUFF_LENGTH];
    char  m_PostIds[POST_ID_MAX_NUM][POST_ID_BUFF_LENGTH];
    nn::fnd::DateTime m_PostDate;
    u8 m_Reserved[1024-984];

    friend class nn::olv::internal::Main;
};

/*!
@brief A class that stores the downloaded topic data.
*/
class DownloadedTopicData
{
public:
    /*!
@brief Instantiates the object.
    */
    DownloadedTopicData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Gets the community ID.

@return Returns the community ID.
    */
    u32 GetCommunityId() const;

private:
    u32 m_CommunityId;
    u8 m_Reserved[64-4];

    friend class nn::olv::internal::Main;
};

/*!
@brief A class that stores the downloaded post data.
*/
class DownloadedPostData : public DownloadedDataBase
{
public:
    /*!
@brief Flags that can be specified in the <tt>@ref TestFlags</tt> function.
    */
    enum
    {
        FLAG_NONE = 0,                                                                      //!< This is the state when no flags are specified.
        FLAG_WITH_BODY_TEXT             = DOWNLOAD_FLAG_VALUE_WITH_BODY_TEXT,               //!< A flag indicating that the post is a text post.
        FLAG_WITH_BODY_MEMO             = DOWNLOAD_FLAG_VALUE_WITH_BODY_MEMO,               //!< A flag indicating that the post is a handwritten memo post.
        FLAG_WITH_EXTERNAL_IMAGE_DATA   = DOWNLOAD_FLAG_VALUE_WITH_EXTERNAL_IMAGE_DATA,     //!< A flag indicating that the post includes an attached image.
        FLAG_WITH_MII_DATA              = DOWNLOAD_FLAG_VALUE_WITH_MII_DATA,                //!< A flag indicating that the post includes Mii character data.
        FLAG_WITH_EXTERNAL_URL          = DOWNLOAD_FLAG_VALUE_WITH_EXTERNAL_URL,            //!< A flag indicating that the post includes an attached URL.
        FLAG_WITH_APP_DATA              = DOWNLOAD_FLAG_VALUE_WITH_APP_DATA,                //!< A flag indicating that the post includes application data.
        FLAG_EMPATHY_ADDED              = DOWNLOAD_FLAG_VALUE_EMPATHY_ADDED,                //!< A flag indicating that you have already given a Yeah to the post.
        FLAG_FREE_FORMAT                = DOWNLOAD_FLAG_VALUE_FREE_FORMAT,                  //!< A flag indicating that the post is a free text post.
        FLAG_SPOILER                    = DOWNLOAD_FLAG_VALUE_SPOILER                       //!< A flag indicating that the post includes spoilers.
    };

    /*!
@brief Instantiates the object.
    */
    DownloadedPostData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Tests flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).

@return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise.
    */
    bool TestFlags(u32 flags) const;

    /*!
@brief Gets the principal ID of the user.

@return Returns the principal ID of the user.
    */
    u32 GetUserPid() const;

    /*!
@brief Gets the date the post was made.

The date returned reflects the difference in the time zone of the account.

@return Returns the date the post was made.
    */
    nn::fnd::DateTime GetPostDate() const;

    /*!
@brief      Gets the mood.

@return Returns the mood.
    */
    s8 GetFeeling() const;

    /*!
@brief Gets the region ID.

@return Returns the region ID.
    */
    u32 GetRegionId() const;

    /*!
@brief Gets the platform ID.

@return Returns the platform ID.
    */
    u8 GetPlatformId() const;

    /*!
@brief Gets the language ID.

See <tt>nn::cfg::CfgLanguageCode</tt> for the language IDs.

@return Returns the language ID.
    */
    u8 GetLanguageId() const;

    /*!
@brief Gets the country ID.

See <tt>nn::cfg::CfgCountryCode</tt> for the country IDs.

@return Returns the country ID.
    */
    u8 GetCountryId() const;

    /*!
@brief Gets the attached URL.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_EXTERNAL_URL</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_EXTERNAL_URL</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">externalUrl</span> argument.
- If a value less than the value of <tt>@ref nn::olv::EXTERNAL_URL_BUFF_LENGTH</tt> is specified for the <span class="argument">externalUrlMaxLength</span> argument.

@param[out] externalUrl             The buffer that stores the attached URL. <br />
@param[in]  externalUrlMaxLength    The length of the buffer that stores the attached URL, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::EXTERNAL_URL_BUFF_LENGTH</tt>.
    */
    void GetExternalUrl(char* externalUrl, u32 externalUrlMaxLength) const;

    /*!
@brief Gets the Mii data of the user.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_MII_DATA</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_MII_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">storeData</span> argument.

@param[out] storeData       The buffer that stores the Mii data. <br />
    */
    void GetMiiData(nn::mii::StoreData* storeData) const;

    /*!
@brief Gets the nickname of the Mii character of the user.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">miiNickname</span> argument.
- If a value less than that of <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt> is specified for the <span class="argument">miiNicknameMaxLength</span> argument.

@param[out] miiNickname             The buffer that stores the nickname of the Mii character. <br />
@param[in]  miiNicknameMaxLength    The length of the buffer that stores the nickname of the Mii character, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt>.
    */
    void GetMiiNickname(wchar_t* miiNickname, u32 miiNicknameMaxLength) const;

    /*!
@brief Gets text data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_BODY_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_BODY_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">bodyText</span> argument.
- If a value less than that of <tt>@ref nn::olv::BODY_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">bodyTextMaxLength</span> argument.

@param[out] bodyText            The buffer that stores the text data.
@param[in]  bodyTextMaxLength   The length of the buffer that stores the text data, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::BODY_TEXT_BUFF_LENGTH</tt>.
    */
    void GetBodyText(wchar_t* bodyText, u32 bodyTextMaxLength) const;

    /*!
@brief Gets the handwritten memo data.

This function can only be used when the <tt>@ref TestFlags</tt> function is set to <tt>@ref FLAG_WITH_BODY_MEMO</tt> and returns <tt>true</tt>. 
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- When the <tt>@ref TestFlags</tt> function is set to <tt>@ref FLAG_WITH_BODY_MEMO</tt> and returns <tt>false</tt>.
- When the <span class="argument">bodyMemo</span> parameter is set to <tt>NULL</tt>.
- When a value less than <tt>@ref nn::olv::BODY_MEMO_BUFF_LENGTH</tt> is specified for the <span class="argument">bodyMemoMaxSize</span> parameter.

@param[out] bodyMemo            The buffer that stores the handwritten memo data.
@param[out] bodyMemoSize        The buffer that stores the size of the handwritten memo data.
@param[in]  bodyMemoMaxSize     The size of the buffer that stores the handwritten memo data. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::BODY_MEMO_BUFF_LENGTH</tt>.
    */
    void GetBodyMemo(u8* bodyMemo, u32* bodyMemoSize, u32 bodyMemoMaxSize) const;

    /*!
@brief      Gets topic tags.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">topicTag</span> argument.
- If a value less than that of <tt>@ref nn::olv::TOPIC_TAG_BUFF_LENGTH</tt> is specified for the <span class="argument">topicTagMaxLength</span> argument.

@param[out] topicTag            The buffer that stores the topic tags.
@param[in]  topicTagMaxLength   The length of the buffer that stores the topic tags, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::TOPIC_TAG_BUFF_LENGTH</tt>.
    */
    void GetTopicTag(wchar_t* topicTag, u32 topicTagMaxLength) const;

    /*!
@brief Gets application data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_APP_DATA</tt> is specified.
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">appData</span> argument.
- If a value less than that returned by the <tt>@ref GetAppDataSize</tt> function is specified for the <span class="argument">appDataMaxSize</span> argument.

@param[out] appData             The buffer that stores the application data.
@param[out] appDataSize         The buffer that stores the size of the application data.
@param[in]  appDataMaxSize      The size of the buffer that stores the application data. <br />
Make sure that you specify a value of at least that returned by the <tt>@ref GetAppDataSize</tt> function.
    */
    void GetAppData(u8* appData, u32* appDataSize, u32 appDataMaxSize) const;

    /*!
@brief Gets the size of the application data.

@return Returns the size of the application data.
    */
    u32 GetAppDataSize() const;

    /*!
@brief Gets the size of the attached image data.

@return Returns the size of the attached image data.
    */
    u32 GetExternalImageDataSize() const;

    /*!
@brief Downloads the attached image data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_EXTERNAL_IMAGE_DATA</tt> is specified. <br />
This function causes Internet communication, during which time processing is blocked. <br />
While this function is being called, you can call the <tt>@ref nn::olv::Cancel</tt> function from another thread to cancel communication processes. <br />
This function is thread safe. However, be careful not to call functions in the OLV library that are not thread safe from other threads while this function is being called.

@param[out] externalImageData            The buffer that stores the attached image data.
@param[out] externalImageDataSize        The buffer that stores the size of the attached image data.
@param[in]  externalImageDataMaxSize     The size of the buffer that stores the attached image data.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
    */
    nn::olv::Result DownloadExternalImageData(void* externalImageData, u32* externalImageDataSize, u32 externalImageDataMaxSize) const;

    /*!
@brief Gets the community ID.

@return Returns the community ID.
    */
    u32 GetCommunityId() const;

    /*!
@brief Gets the Yeah count.

@return Returns the Yeah count.
    */
    u32 GetEmpathyCount() const;

    /*!
@brief Gets the number of comments.

@return Returns the number of comments.
    */
    u32 GetCommentCount() const;

    /*!
@brief Gets the post ID.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">postId</span> argument.
- If a value less than that of <tt>@ref nn::olv::POST_ID_BUFF_LENGTH</tt> is specified for the <span class="argument">postIdMaxLength</span> argument.

@param[out] postId              The buffer that stores the post ID.
@param[in]  postIdMaxLength     The length of the buffer that stores the post ID, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::POST_ID_BUFF_LENGTH</tt>.
    */
    void GetPostId(char* postId, u32 postIdMaxLength) const;

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
