/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_TypesDownloadUserData.h

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_DOWNLOAD_USERDATA_H_
#define __OLV_TYPES_DOWNLOAD_USERDATA_H_

/*! @file
    @brief      OLV type declarations related to the downloading of user data.
*/

#include <nn/olv/olv_TypesDownloadedDataBase.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
    @brief The class representing the parameters used when downloading user data.
*/
class DownloadUserDataListParam
{
public:
    /*!
        @brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.

        A flag other than <tt>FLAG_NONE</tt> must be specified. <br />
        Only one flag can be specified.
    */
    enum
    {
        FLAG_NONE                  = 0,                //!< This is the state when no flags are specified.
        FLAG_FRIEND_TO_USER        = (1<<0),           //!< Downloads user data from friends of the specified user. Specify the target user with the <tt>@ref SetUserPid</tt> function.
        FLAG_FOLLOW_TO_USER        = (1<<1),           //!< Downloads user data from the users that the specified user is following. Specify the target user with the <tt>@ref SetUserPid</tt> function.
        FLAG_EMPATHY_TO_POST       = (1<<2),           //!< Downloads user data from users who have given a Yeah to the specified post. Specify the target post with the <tt>@ref SetPostId</tt> function.
        FLAG_FAVORITE_TO_COMMUNITY = (1<<3),           //!< Downloads user data from users who have registered the specified community as a favorite. Specify the target community with the <tt>@ref SetCommunityId</tt> function.
        FLAG_WITH_MII_DATA         = (1<<4)            //!< Downloads user data including the Mii data.
    };

    /*!
        @brief Instantiates the object.
    */
    DownloadUserDataListParam();

    /*!
        @brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
        @brief Sets flags.

        You must specify this option.

        @param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
        @brief Sets the principal ID.

        Specifying this option is not required. <br />
        Specify <tt>@ref FLAG_FRIEND_TO_USER</tt> or <tt>@ref FLAG_FOLLOW_TO_USER</tt> at the same time. <br />
        When the principal ID for some user is specified, the user data is downloaded for the friends of that user or the people being followed by that user. <br />

        @param[in]  userPid The principal ID. <br />
    */
    void SetUserPid(u32 userPid);

    /*!
        @brief Sets the post ID.

        Specifying this option is not required. <br />
        Specify <tt>@ref FLAG_EMPATHY_TO_POST</tt> at the same time. <br />
        When this is specified, the user data is downloaded for users who have given a Yeah to the specified post. <br />

        @param[in]  postId      The post ID. <br />
                                The maximum number of characters that can be specified is <tt>@ref nn::olv::POST_ID_BUFF_LENGTH - 1</tt>.
    */
    void SetPostId(const char* postId);

    /*!
        @brief Sets the community ID.

        Specifying this option is not required. <br />
        Specify <tt>@ref FLAG_FAVORITE_TO_COMMUNITY</tt> at the same time. <br/>
        When this is specified, the user data is downloaded for users who have registered the specified community as a favorite. <br />

        @param[in]  communityId     The community ID. <br />
    */
    void SetCommunityId(const u32 communityId);

    /*!
        @brief Sets the maximum amount of user data to download.

        You must specify this option.

        @param[in]  userDataMaxNum  The maximum amount of user data to download. <br />
                                    Make sure that the number of arrays in the <span class="argument">userDataList</span> parameter of the <tt>@ref nn::olv::DownloadUserDataList</tt> function is equal to or greater than this value. <br />
    */
    void SetUserDataMaxNum(u32 userDataMaxNum);

    /*!
        @brief Gets the size of the working buffer to use with the <tt>@ref nn::olv::DownloadUserDataList</tt> function.

        Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
        This function returns fixed values.

        @return Returns the size of the working buffer to use with the <tt>@ref nn::olv::DownloadUserDataList</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
        @brief Specifies the size of the working buffer to use with the <tt>@ref nn::olv::DownloadUserDataList</tt> function.

        You must specify this option. <br />
        Specify a buffer of at least the size obtained with the <tt>@ref GetWorkSize</tt> function. <br />

        @param[in]  work       The working buffer.
        @param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_UserPid;
    u32 m_CommunityId;
    s8  m_IsSetCommunityId;
    NN_PADDING3;
    u32 m_UserDataMaxNum;
    char  m_PostId[POST_ID_BUFF_LENGTH];
    u8  m_Reserved[1024-60];
 
    friend class nn::olv::internal::Main;
};

/*!
    @brief The class for storing the downloaded user data.
*/
class DownloadedUserData
{
public:
    /*!
        @brief Flags that can be specified in the <tt>@ref TestFlags</tt> function.
    */
    enum
    {
        FLAG_NONE = 0,                                           //!< This is the state when no flags are specified.
        FLAG_WITH_MII_DATA = DOWNLOAD_FLAG_VALUE_WITH_MII_DATA   //!< This flag indicates that the user data includes Mii data.
    };

    /*!
        @brief Instantiates the object.
    */
    DownloadedUserData();

    /*!
        @brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
        @brief Tests flags.

        @param[in] flags  Flags (<tt>FLAG_*</tt>).

        @return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise.
    */
    bool TestFlags(u32 flags) const;

    /*!
        @brief Gets the principal ID of the user.

        @return Returns the principal ID of the user.
    */
    u32 GetUserPid() const;

    /*!
        @brief Gets the Mii data of the user.

        This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_MII_DATA</tt> is specified. <br />
        <br />
        Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
        - If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_MII_DATA</tt> specified yields a return value of <tt>false</tt>.
        - If <tt>NULL</tt> is specified for the <span class="argument">storeData</span> argument.

        @param[out] storeData       The buffer that stores the Mii data. <br />
    */
    void GetMiiData(nn::mii::StoreData* storeData) const;

    /*!
        @brief Gets the nickname of the Mii of the user.

        Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
        - If <tt>NULL</tt> is specified for the <span class="argument">miiNickname</span> argument.
        - If a value less than that of <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt> is specified for the <span class="argument">miiNicknameMaxLength</span> argument.

        @param[out] miiNickname             The buffer that stores the nickname of the Mii. <br />
        @param[in]  miiNicknameMaxLength    The length of the buffer that stores the nickname of the Mii, including the terminating null character. <br />
                                            Make sure that you specify a size of at least <tt>@ref nn::olv::MII_NICKNAME_BUFF_LENGTH</tt>.
    */
    void GetMiiNickname(wchar_t* miiNickname, u32 miiNicknameMaxLength) const;

private:
    u32 m_Flags;
    u32 m_UserPid;
    u8 m_MiiData[MII_DATA_BUFF_LENGTH];
    wchar_t m_MiiNickname[MII_NICKNAME_BUFF_LENGTH];
    u8  m_Reserved[2048-168];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
