/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesResume.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_RESUME_H_
#define __OLV_TYPES_RESUME_H_

/*! @file
@brief      Declaration of types related to OLV recovery.
*/

#include <nn/config.h>
#include <nn/types.h>
#include <nn/olv/olv_Const.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief Represents the parameters when recovering.
*/
class ResumeParam
{
public:
    /*!
@brief Instantiates the object.
    */
    ResumeParam();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

This is for future expansion. It is not used currently.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief      Sets the system arguments.

The system arguments can be obtained by using the <tt>nn::applet::GetInitialParam</tt> function when the application starts, and by using the <tt>nn::applet::WaitForStarting</tt> function when you return from the Miiverse application. For an example of how these are used, see the <tt>StartPortalApp</tt> demo.

@param[in]  systemArgs      The system arguments.
@param[in]  systemArgsSize  The size of the system arguments.
    */
    void SetSystemArgs(const u8* systemArgs, u32 systemArgsSize);

private:
    u32 m_Flags;
    const u8* m_SystemArgs;
    u32 m_SystemArgsSize;
    u8 m_Reserved[64-12];

    friend class nn::olv::internal::Main;
};

/*!
@brief A class that stores the recovery results.
*/
class ResumedData
{
public:
    /*!
@brief The modes that can be obtained using the <tt>GetMode</tt> function.
    */
    enum {
        MODE_NONE = 0,                                          //!< The default mode.
        MODE_START_COMMUNITY    = MODE_VALUE_START_COMMUNITY,   //!< The mode specified when the process is started from the community page.
        MODE_START_POST         = MODE_VALUE_START_POST,        //!< The mode specified when the process is started from the post page.
        MODE_START_USER         = MODE_VALUE_START_USER         //!< The mode specified when the process is started from the user page.
    };

    /*!
@brief Flags that can be specified in the <tt>TestFlags</tt> function.
    */
    enum
    {
        FLAG_NONE = 0,                                                          //!< This is the state when no flags are specified.
        FLAG_WITH_APP_DATA              = DOWNLOAD_FLAG_VALUE_WITH_APP_DATA     //!< Flag indicating that application data is included.
    };

    /*!
@brief Instantiates the object.
    */
    ResumedData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Gets the mode.

@return Returns the mode (<tt>MODE_*</tt>).
    */
    u32 GetMode() const;

    /*!
@brief Tests flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).

@return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise. <br />
    */
    bool TestFlags(u32 flags) const;

    /*!
@brief Gets the community ID.

This function only can be used when the <tt>@ref GetMode</tt> function returns <tt>@ref MODE_START_COMMUNITY</tt>. <br />

@return Returns the community ID.
    */
    u32 GetCommunityId() const;

    /*!
@brief Gets the post ID.

This function only can be used when the <tt>@ref GetMode</tt> function returns <tt>@ref MODE_START_POST</tt>. <br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">postId</span> argument.
- If a value less than that of <tt>@ref nn::olv::POST_ID_BUFF_LENGTH</tt> is specified for the <span class="argument">postIdMaxLength</span> argument.

@param[out] postId              The buffer that stores the post ID.
@param[in]  postIdMaxLength     The length of the buffer that stores the post ID, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::POST_ID_BUFF_LENGTH</tt>.
    */
    void GetPostId(char* postId, u32 postIdMaxLength) const;

    /*!
@brief Gets the size of the application data.

@return Returns the size of the application data.
    */
    u32 GetAppDataSize() const;

    /*!
@brief Gets application data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_APP_DATA</tt> is specified.
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">appData</span> argument.
- If a value less than that returned by the <tt>@ref GetAppDataSize</tt> function is specified for the <span class="argument">appDataMaxSize</span> argument.

@param[out] appData             The buffer that stores the application data.
@param[in]  appDataMaxSize      The size of the buffer that stores the application data. <br />
Make sure that you specify a value of at least that returned by the <tt>@ref GetAppDataSize</tt> function.
    */
    void GetAppData(u8* appData, u32 appDataMaxSize) const;

    /*!
@brief Gets the principal ID of the user.

This function only can be used when the <tt>@ref GetMode</tt> function returns <tt>@ref MODE_START_USER</tt>. <br />

@return Returns the principal ID of the user.
    */
    u32 GetUserPid() const;

private:
    u32 m_Mode;
    u32 m_Flags;
    u32 m_CommunityId;
    u8* m_PostId;
    u32 m_PostIdLength;
    u8* m_AppData;
    u32 m_AppDataSize;
    u32 m_UserPid;
    u8 m_CommonData[4096-32];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
