/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesStartPortalApp.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_START_PORTALAPP_H_
#define __OLV_TYPES_START_PORTALAPP_H_

/*! @file
@brief      Declaration of types related to starting the OLV Miiverse application.
*/

#include <nn/config.h>
#include <nn/types.h>
#include <nn/olv/olv_Const.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief Represents the parameters when starting the Miiverse application.
*/
class StartPortalAppParam
{
public:

    /*!
@brief Modes that can be specified in the <tt>SetMode</tt> function.
    */
    enum {
        MODE_NONE = 0,                                          //!< The default mode.
        MODE_START_COMMUNITY    = MODE_VALUE_START_COMMUNITY,   //!< The mode to specify when showing a community.
        MODE_START_POST         = MODE_VALUE_START_POST,        //!< The mode to specify when showing a post.
        MODE_START_USER         = MODE_VALUE_START_USER         //!< The mode to specify when showing a user.
    };

    /*!
@brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.
    */
    enum {
        FLAG_NONE = 0                               //!< This is the state when no flags are specified.
    };

    /*!
@brief Constant.
    */
    enum {
        COMMON_DATA_MAX_NUM = 256                   //!< The maximum amount of common data that can added.
    };

    /*!
@brief Instantiates the object.
    */
    StartPortalAppParam(); 

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets the mode.

Specifying this option is not required.

@param[in]  mode        The mode (<tt>MODE_*</tt>).
    */
    void SetMode(u32 mode);

    /*!
@brief Sets flags.

This is for future expansion. It is not used currently.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the post ID.

You must specify this option when <tt>@ref MODE_START_POST</tt> is specified for the <tt>@ref SetMode</tt> function. <br />
Note that the only value stored is the address, so the data must remain valid until the call to the <tt>@ref nn::olv::StartPortalApp</tt> function ends.

@param[in]  postId      The post ID. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::POST_ID_BUFF_LENGTH - 1</tt>.
    */
    void SetPostId(const char* postId);

    /*!
@brief Sets the community ID.

You can specify this option when <tt>@ref MODE_START_COMMUNITY</tt> is specified for the <tt>@ref SetMode</tt> function. If not specified, the general community is shown by default.

@param[in]  communityId     The community ID.
    */
    void SetCommunityId(u32 communityId);

    /*!
@brief Sets the principal ID.

You must specify this option when <tt>@ref MODE_START_USER</tt> is specified for the <tt>@ref SetMode</tt> function.

@param[in]  userPid The principal ID.
    */
    void SetUserPid(u32 userPid);

    /*!
@brief Sets the topic tag.

Specifying this option is not required. <br />
You can only specify this option when <tt>@ref MODE_START_COMMUNITY</tt> is specified for the <tt>@ref SetMode</tt> function. <br />
When specified, only posts with the specified topic tag are shown. <br />
Note that the only value stored is the address, so the data must remain valid until the call to the <tt>@ref nn::olv::StartPortalApp</tt> function ends.

@param[in]  topicTag    The topic tag. <br />
Use UTF-16LE for the encoding. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::TOPIC_TAG_MAX_LENGTH</tt>.
    */
    void SetTopicTag(const wchar_t* topicTag);

    /*!
@brief Adds common data.

Specifying this option is not required. <br />
Under normal use conditions this specification is not required. <br />
The maximum amount that can be added is <tt>nn::olv::StartPortalAppParam::COMMON_DATA_MAX_NUM</tt>. <br />
Note that the only value stored is the address, so the data must remain valid until the call to the <tt>@ref nn::olv::StartPortalApp</tt> function ends.

@param[in]  type        The type of the common data (COMMON_DATA_TYPE_*).
@param[in]  data        The common data.
@param[in]  dataSize    The size of the common data.
    */
    void SetCommonData(u32 type, const u8* data, u32 dataSize);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::StartPortalApp</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
- The required size will increase according to the specified parameters.

@return Returns the size of the working buffer.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the size of the working buffer used by the <tt>@ref nn::olv::StartPortalApp</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>GetWorkSize</tt> function. <br />
Device memory cannot be specified for this buffer. <br />
This buffer cannot be destroyed at the point the call to the <tt>@ref nn::olv::StartPortalApp</tt> function finishes. See the description of the <tt>@ref nn::olv::StartPortalApp</tt> function.

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:

    u32 m_Mode;
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_CommonDataNum;
    u32 m_CommonDataType[COMMON_DATA_MAX_NUM];
    const u8* m_CommonData[COMMON_DATA_MAX_NUM];
    u32 m_CommonDataSize[COMMON_DATA_MAX_NUM];
    bool m_CommonDataIsValue[COMMON_DATA_MAX_NUM];
    u8 m_Reserved[4096-3348];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
