/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesUploadCommunityData.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_UPLOAD_COMMUNITYDATA_H_
#define __OLV_TYPES_UPLOAD_COMMUNITYDATA_H_

/*! @file
@brief      Declaration of types related to uploading OLV community data.
*/

#include <nn/config.h>
#include <nn/types.h>
#include <nn/mii/mii_StoreData.h>
#include <nn/olv/olv_Const.h>
#include <nn/olv/olv_Result.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief The class representing the parameters used when creating, updating, and deleting community data.
*/
class UploadCommunityDataParam
{
public:
    /*!
@brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.
    */
    enum
    {
        FLAG_NONE = 0,                                     //!< This is the state when no flags are specified.
        FLAG_DELETE     = UPLOAD_FLAG_VALUE_DELETE         //!< Specify this flag when deleting community data.
    };

    /*!
@brief Instantiates the object.
    */
    UploadCommunityDataParam();


    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the community ID.

You must specify this option when updating or deleting community data. If not specified, new community data is created by default.

@param[in]  communityId     The community ID.
    */
    void SetCommunityId(u32 communityId);

    /*!
@brief Sets the community name.

You must specify this option when creating new community data.

@param[in]  titleText   The community name. <br />
Use UTF-16LE for the encoding. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::TITLE_TEXT_BUFF_LENGTH - 1</tt>.
    */
    void SetTitleText(const wchar_t* titleText);

    /*!
@brief Sets the community description.

Specifying this option is not required.

@param[in]  descriptionText     The community description. <br />
Use UTF-16LE for the encoding. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH - 1</tt>.
    */
    void SetDescriptionText(const wchar_t* descriptionText);

    /*!
@brief Specifies the maximum number of characters that can be posted from the Miiverse application.

Specifying this option is not required. <br />
If nothing is specified, 100 characters is set as the default.

@param[in]  length              The maximum number of characters that can be posted from the Miiverse application. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::BODY_TEXT_MAX_LENGTH</tt>.
    */
    void SetPostTextMaxLength(u32 length);

    /*!
@brief Specifies the maximum number of characters for comments that can be posted from the Miiverse application.

Specifying this option is not required. <br />
If nothing is specified, 100 characters is set as the default.

@param[in]  length              The maximum number of characters for comments posted from the Miiverse application. <br />
The maximum number of characters that can be specified is <tt>@ref nn::olv::BODY_TEXT_MAX_LENGTH</tt>.
    */
    void SetCommentTextMaxLength(u32 length);

    /*!
@brief Sets the icon data.

Specifying this option is not required.

@param[in]  iconData        The icon data. <br />
Note that only the address value is stored, so this data must be valid until the call to the <tt>@ref nn::olv::UploadCommunityData</tt> function ends. <br />
@param[in]  iconDataSize    The size of the icon data. <br />
The maximum value that can be specified is <tt>@ref nn::olv::ICON_DATA_BUFF_LENGTH</tt>.
    */
    void SetIconData(const u8* iconData, u32 iconDataSize);

    /*!
@brief Sets the application data.

Specifying this option is not required.

@param[in]  appData     The application data. <br />
Note that only the address value is stored, so this data must be valid until the call to the <tt>@ref nn::olv::UploadCommunityData</tt> function ends.
@param[in]  appDataSize The size of the application data. <br />
The maximum value that can be specified is <tt>@ref nn::olv::APP_DATA_BUFF_LENGTH</tt>.
    */
    void SetAppData(const u8* appData, u32 appDataSize);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::UploadCommunityData</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
This function returns fixed values.

@return Returns the size of the working buffer used by the <tt>@ref nn::olv::UploadCommunityData</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the size of the working buffer used by the <tt>@ref nn::olv::UploadCommunityData</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>@ref GetWorkSize</tt> function. <br />

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_CommunityId;
    bool m_IsSetCommunityId;
    NN_PADDING3;
    wchar_t m_TitleText[TITLE_TEXT_BUFF_LENGTH];
    wchar_t m_DescriptionText[DESCRIPTION_TEXT_BUFF_LENGTH];
    const u8* m_AppData;
    u32 m_AppDataSize;
    const u8* m_IconData;
    u32 m_IconDataSize;
    u32 m_PostTextMaxLength;
    u32 m_CommentTextMaxLength;
    u8 m_Reserved[1024-812];

    friend class nn::olv::internal::Main;
};

/*!
@brief The class for storing the results when community data is created or uploaded.
*/
class UploadedCommunityData
{
public:
    /*!
@brief Flags that can be specified in the <tt>TestFlags</tt> function.
    */
    enum Flags
    {
        FLAG_NONE = 0,                                                              //!< This is the state when no flags are specified.
        FLAG_WITH_TITLE_TEXT        = DOWNLOAD_FLAG_VALUE_WITH_TITLE_TEXT,          //!< A flag indicating that the community name is included in the community data.
        FLAG_WITH_DESCRIPTION_TEXT  = DOWNLOAD_FLAG_VALUE_WITH_DESCRIPTION_TEXT,    //!< A flag indicating that the community description is included in the community data.
        FLAG_WITH_APP_DATA          = DOWNLOAD_FLAG_VALUE_WITH_APP_DATA,            //!< A flag indicating that application data is included in the community data.
        FLAG_WITH_ICON_DATA         = DOWNLOAD_FLAG_VALUE_WITH_ICON_DATA            //!< A flag indicating that icon data is included in the community data.
    };

    /*!
@brief Instantiates the object.
    */
    UploadedCommunityData();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Tests flags.

@param[in] flags  Flags (<tt>FLAG_*</tt>).

@return Returns <tt>true</tt> if any of the flags are set; returns <tt>false</tt> otherwise.
    */
    bool TestFlags(u32 flags) const;

    /*!
@brief Gets the community ID.

@return Returns the community ID.
    */
    u32 GetCommunityId() const;

    /*!
@brief Gets the community code.

Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If <tt>NULL</tt> is specified for the <span class="argument">communityCode</span> argument.
- If a value less than that of <tt>@ref nn::olv::COMMUNITY_CODE_BUFF_LENGTH</tt> is specified for the <span class="argument">communityCodeMaxLength</span> argument.

@param[out] communityCode               The buffer for storing the community code.
@param[in]  communityCodeMaxLength      The length of the buffer that stores the community code, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::COMMUNITY_CODE_BUFF_LENGTH</tt>.
    */
    void GetCommunityCode(char* communityCode, u32 communityCodeMaxLength) const;

    /*!
@brief Gets the principal ID of the user who created the community.

@return Returns the principal ID of the user who created the community.
    */
    u32 GetOwnerPid() const;

    /*!
@brief Gets the community name.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_TITLE_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_TITLE_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">titleText</span> argument.
- If a value less than that of <tt>@ref nn::olv::TITLE_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">titleTextMaxLength</span> argument.

@param[out] titleText           The buffer for storing the community name.
@param[in]  titleTextMaxLength  The length of the buffer that stores the community name, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::TITLE_TEXT_BUFF_LENGTH</tt>.
    */
    void GetTitleText(wchar_t* titleText, u32 titleTextMaxLength) const;

    /*!
@brief Gets the community description.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_DESCRIPTION_TEXT</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_DESCRIPTION_TEXT</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument>descriptionText</span> argument.
- If a value less than that of <tt>@ref nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH</tt> is specified for the <span class="argument">descriptionTextMaxLength</span> argument.

@param[out] descriptionText             The buffer for storing the community description.
@param[in]  descriptionTextMaxLength    The length of the buffer that stores the community description, including the terminating null character. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH</tt>.
    */
    void GetDescriptionText(wchar_t* descriptionText, u32 descriptionTextMaxLength) const;

    /*!
@brief Gets application data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_APP_DATA</tt> is specified. <br />
<br />
Note that if you call this function in one of the following invalid states, it will stop on an assertion in the debug/development versions, and return without doing anything in the release version. <br />
- If calling the <tt>@ref TestFlags</tt> function with <tt>@ref FLAG_WITH_APP_DATA</tt> specified yields a return value of <tt>false</tt>.
- If <tt>NULL</tt> is specified for the <span class="argument">appData</span> argument.
- If a value less than that returned by the <tt>@ref GetAppData</tt> function is specified for the <span class="argument">appDataMaxSize</span> argument.

@param[out] appData             The buffer that stores the application data.
@param[out] appDataSize         The buffer that stores the size of the application data.
@param[in]  appDataMaxSize      The size of the buffer that stores the application data. <br />
Make sure that you specify a value of at least that returned by the <tt>@ref GetAppData</tt> function.
    */
    void GetAppData(u8* appData, u32* appDataSize, u32 appDataMaxSize) const;

    /*!
@brief Gets the size of the application data.

@return Returns the size of the application data.
    */
    u32 GetAppDataSize(void) const;

    /*!
@brief Gets icon data.

This function can only be used when the return value from the <tt>@ref TestFlags</tt> function is <tt>true</tt> when <tt>@ref FLAG_WITH_ICON_DATA</tt> is specified.

@param[out] iconData            The buffer that stores the icon data.
@param[out] iconDataSize        The buffer that stores the size of the icon data.
@param[in]  iconDataMaxSize     The size of the buffer that stores the icon data. <br />
Make sure that you specify a size of at least <tt>@ref nn::olv::ICON_DATA_BUFF_LENGTH</tt>.

@return Returns the result of the operation. <br />
For information about how to handle the results, see the OLV library reference.
    */
    nn::olv::Result GetIconData(u8* iconData, u32* iconDataSize, u32 iconDataMaxSize) const;

private:
    u32 m_Flags;
    u32 m_CommunityId;
    u32 m_OwnerPid;
    wchar_t m_TitleText[TITLE_TEXT_BUFF_LENGTH];
    u32 m_TitleTextLength;
    wchar_t m_DescriptionText[DESCRIPTION_TEXT_BUFF_LENGTH];
    u32 m_DescriptionTextLength;
    u8  m_AppData[APP_DATA_BUFF_LENGTH];
    u32 m_AppDataSize;
    u8  m_IconData[COMPRESSED_ICON_DATA_BUFF_LENGTH];
    s32 m_IconDataSize;
    u8  m_MiiData[MII_DATA_BUFF_LENGTH];
    wchar_t m_MiiNickname[MII_NICKNAME_BUFF_LENGTH];
    u8 m_Reserved[12288-11240];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
