/*---------------------------------------------------------------------------*
Project: OLV
File: olv_TypesUploadFavoriteToCommunityData.h

Copyright (C) 2009-2013 Nintendo. All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#ifndef __OLV_TYPES_UPLOAD_FAVORITE_TO_COMMUNITYDATA_H_
#define __OLV_TYPES_UPLOAD_FAVORITE_TO_COMMUNITYDATA_H_

/*! @file
@brief      Declaration of types related to registering or removing OLV communities as favorites.
*/

#include <nn/config.h>
#include <nn/types.h>
#include <nn/olv/olv_Const.h>

/// <span>nn</span>
namespace nn {

/// <span>olv</span>
namespace olv {

namespace internal 
{
class Main;
}

/*/* @defgroup class Classes
 *  @{
 */

/*!
@brief The class representing the parameters used when registering a favorite to a community or removing a favorite.
*/
class UploadFavoriteToCommunityDataParam
{
public:
    /*!
@brief Flags that can be specified in the <tt>@ref SetFlags</tt> function.
    */
    enum Flags
    {
        FLAG_NONE = 0,                      //!< This is the state when no flags are specified.
        FLAG_REMOVE = (1<<0)                //!< Specify this flag when removing a favorite.
    };

    /*!
@brief Instantiates the object.
    */
    UploadFavoriteToCommunityDataParam();

    /*!
@brief Reverts parameters to their initial state.
    */
    void Reset();

    /*!
@brief Sets flags.

Specifying this option is not required.

@param[in] flags  Flags (<tt>FLAG_*</tt>).
    */
    void SetFlags(u32 flags);

    /*!
@brief Sets the community ID.

Specifying this option is not required. You are prohibited from specifying this option when a community code is specified for the <tt>@ref SetCommunityCode</tt> function. <br />
If neither a community ID nor a community code is specified, the general community is registered or removed as a favorite by default.

@param[in]  communityId     The community ID.
    */
    void SetCommunityId(u32 communityId);

    /*!
@brief Sets the community code.

Specifying this option is not required. You are prohibited from specifying this option when a community ID is specified for the <tt>@ref SetCommunityId</tt> function. <br />
If neither a community ID nor a community code is specified, the general community is registered or removed as a favorite by default.

@param[in]  communityCode   The community code.
    */
    void SetCommunityCode(const char* communityCode);

    /*!
@brief Gets the size of the working buffer used by the <tt>@ref nn::olv::UploadFavoriteToCommunityData</tt> function.

Call this function after all of the various parameters have been specified, with the exception of the <tt>@ref SetWork</tt> function. <br />
This function returns fixed values.

@return Returns the size of the working buffer used by the <tt>@ref nn::olv::UploadFavoriteToCommunityData</tt> function.
    */
    u32 GetWorkSize() const;

    /*!
@brief Sets the working buffer used by the <tt>@ref nn::olv::UploadFavoriteToCommunityData</tt> function.

You must specify this option. <br />
Specify a buffer of at least the size obtained with the <tt>@ref GetWorkSize</tt> function. <br />

@param[in]  work       The working buffer.
@param[in]  workSize   The size of the working buffer.
    */
    void SetWork(u8* work, u32 workSize);

private:
    u32 m_Flags;
    u8* m_Work;
    u32 m_WorkSize;
    u32 m_CommunityId;
    s8 m_IsSetCommunityId;
    NN_PADDING3;
    u8 m_Reserved[64-20];

    friend class nn::olv::internal::Main;
};

/** @} */

}   // namespace olv
}   // namespace nn
#endif
