﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_DownloadCommentDataList_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of downloading comment data.
 DOWNLOAD_MAX_NUM of post data is downloaded, and post IDs are stored for those post data that have one or more comments.
 These post IDs are then used to download the comment data, and the information is displayed.
 *---------------------------------------------------------------------------
*/
#include <nn.h>
#include <nn/olv.h>
#include <nn/act/act_Api.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for the post data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadedPostData s_postDataList[DOWNLOAD_MAX_NUM];
    nn::olv::DownloadedTopicData s_topicData;

    // Set a post ID for the following string to download comments about that post.
    // Set an empty string to download post data and automatically get the post IDs.
    char s_TargetPostId[] = "";

    // Container for the comment data.
    nn::olv::DownloadedCommentData s_commentDataList[DOWNLOAD_MAX_NUM];
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "DownloadCommentDataList demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

void PrintDownloadedCommentData(const nn::olv::DownloadedCommentData& param)
{
    NN_LOG("-------------------------------\n");
    NN_LOG("print DownloadedCommentData info:\n");

    // The date and time.
    nn::fnd::DateTime date = param.GetPostDate();
    OLVDemo_PrintDate(date);

    // Do flag test and get value.
    if (param.TestFlags(nn::olv::DownloadedCommentData::FLAG_WITH_BODY_TEXT))
    {
        NN_LOG("text data ...\n");
        wchar_t buff[nn::olv::BODY_TEXT_BUFF_LENGTH];
        param.GetBodyText(buff, nn::olv::BODY_TEXT_BUFF_LENGTH);
        OLVDemo_PrintWideStr(buff, 0);
    }

    if (param.TestFlags(nn::olv::DownloadedCommentData::FLAG_WITH_BODY_MEMO))
    {
        NN_LOG("memo data ...\n");
        OLVDemo_MemoryManager data;
        if (data.Allocate(nn::olv::BODY_MEMO_BUFF_LENGTH))
        {   // Display partially.
            u32 dataSize;
            param.GetBodyMemo(data.buf, &dataSize, nn::olv::BODY_MEMO_BUFF_LENGTH);
            NN_LOG("memo size:%d\n", dataSize);
            OLVDemo_PrintBinary(data.buf, dataSize > 1024 ? 1024 : dataSize);
        }
    }

    if (param.TestFlags(nn::olv::DownloadedCommentData::FLAG_WITH_APP_DATA))
    {
        NN_LOG("app data size:%d ...\n", param.GetAppDataSize());
        OLVDemo_MemoryManager data;
        if (data.Allocate(param.GetAppDataSize()))
        {
            u32 dataSize;
            param.GetAppData(data.buf, &dataSize, param.GetAppDataSize());
            NN_LOG("appData size:%d\n", dataSize);
            OLVDemo_PrintBinary(data.buf, dataSize);
        }
    }

    if (param.TestFlags(nn::olv::DownloadedCommentData::FLAG_WITH_EXTERNAL_IMAGE_DATA))
    {
        NN_LOG("external image data ...\n", param.GetExternalImageDataSize());
        OLVDemo_MemoryManager data;
        if (data.Allocate(param.GetExternalImageDataSize()))
        {   // Display partially.
            u32 dataSize;
            param.DownloadExternalImageData(data.buf, &dataSize, param.GetExternalImageDataSize());
            NN_LOG("external image data size:%d\n", dataSize);
            OLVDemo_PrintBinary(data.buf, dataSize > 1024 ? 1024 : dataSize);
        }
    }

    if (param.TestFlags(nn::olv::DownloadedCommentData::FLAG_WITH_MII_DATA))
    {
        nnmiiStoreData miiData;
        wchar_t miiNickName[nn::olv::MII_NICKNAME_BUFF_LENGTH];
        NN_LOG("mii data ...\n");
        param.GetMiiData(&miiData);
        param.GetMiiNickname(miiNickName, nn::olv::MII_NICKNAME_BUFF_LENGTH);
        OLVDemo_PrintWideStr(miiNickName);
    }

    // Comment ID.
    char commentId[nn::olv::COMMENT_ID_BUFF_LENGTH];
    param.GetCommentId(commentId, nn::olv::COMMENT_ID_BUFF_LENGTH);
    NN_LOG("comment Id:%s\n", commentId);

    // Other.
    NN_LOG("feeling:%d\n", param.GetFeeling());
    NN_LOG("pid:%d\n", param.GetUserPid());
    NN_LOG("region:%d\n", param.GetRegionId());
    NN_LOG("platformId:%d\n", param.GetPlatformId());
    NN_LOG("languageid:%d\n", param.GetLanguageId());
    NN_LOG("countryId:%d\n", param.GetCountryId());
    NN_LOG("spoiler:%s\n", param.TestFlags(nn::olv::DownloadedCommentData::FLAG_SPOILER) ? "true" : "false");
    NN_LOG("free format:%s\n", param.TestFlags(nn::olv::DownloadedCommentData::FLAG_FREE_FORMAT) ? "true" : "false");
    NN_LOG("-------------------------------\n\n");
}

// Get the post ID for the post data to download the comment data.
bool GetPostIdFromDownloadData(char *postIdBuffer, const u32 postIdBufferSize)
{
    nn::olv::Result result;

    nn::olv::DownloadPostDataListParam downloadParam;
    
    downloadParam.SetPostDataMaxNum(DOWNLOAD_MAX_NUM);
 
    // Set the flag
    u32 flags = 0;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_BODY_TEXT;
    downloadParam.SetFlags(flags);
 
    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadPostDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the post data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadPostDataList(
            &s_topicData, s_postDataList,
            &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadPostDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }

    // Find and store the first post data that contains one or more comments.
    NN_LOG("downloaded num:%d\n", downloadListSize);
    for (s32 i = 0; i < downloadListSize; ++i)
    {
        if (s_postDataList[i].GetCommentCount() > 0)
        {
            char postIdTmpBuff[nn::olv::POST_ID_BUFF_LENGTH];
            s_postDataList[i].GetPostId(postIdTmpBuff, nn::olv::POST_ID_BUFF_LENGTH);
            strncpy(postIdBuffer, postIdTmpBuff, postIdBufferSize);
            NN_LOG("GetPostIdFromDownloadData success. postId is %s.\n", postIdBuffer);
            return true;
        }
    }

    // If none of the downloaded post data have any comments, the process arrives here and the demo fails.
    // 
    // In that case, either use the Miiverse application to send the comment data, or try again after, for example, increasing the value of DOWNLOAD_MAX_NUM.
    // 
    NN_LOG("GetPostIdFromDownloadData failed. no commented posts.\n");

    return false;
}

// Sample functions for downloading comment data.
void DownloadCommentDataListSample(const char* postId)
{
    nn::olv::Result result;
    
    // Parameter when downloading comment data.
    nn::olv::DownloadCommentDataListParam downloadParam;
    
    // Set the ID of the post for which comment data is being downloaded.
    downloadParam.SetPostId(postId);
  
    // Set the maximum number to download.
    downloadParam.SetCommentDataMaxNum(DOWNLOAD_MAX_NUM);
    
    u32 flags = 0;    
    // Set the data body type.
    // If this flag is not set, then the data will be downloaded with no distinction between text and handwritten memos.
    //flags |= nn::olv::DownloadCommentDataListParam::FLAG_FILTER_BY_BODY_TEXT;
    //flags |= nn::olv::DownloadCommentDataListParam::FLAG_FILTER_BY_BODY_MEMO;
    // Download comment data including Mii character data.
    flags |= nn::olv::DownloadCommentDataListParam::FLAG_WITH_MII_DATA;
    // Download only one comment from each poster.
    //flags |= nn::olv::DownloadCommentDataListParam::FLAG_DISTINCT_PID;
    // Download comments including spoilers.
    //flags |= nn::olv::DownloadCommentDataListParam::FLAG_WITH_SPOILER;

    downloadParam.SetFlags(flags);

    // Receive comments, specifying the send date and time.
    // Set the date and time in the format of nn::fnd::DateTime.
    //nn::fnd::DateTime date (2013, 1, 1, 0, 0, 0);
    //downloadParam.SetPostDate(date);
    
    // Set a language ID if you want to filter for only a certain language.
    //downloadParam.SetLanguageId(nn::olv::LANGUAGE_ID_ALL);
    
    // If you want to download only certain comment data, specify the comment ID.
    // When specifying a comment ID, the only option that you can set is FLAG_WITH_MII_DATA.
    // 
    //downloadParam.SetCommentId("");
    
    // Set the maximum number of characters of text data.
    downloadParam.SetBodyTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadCommentDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workMemory.buf, workSize);

    // Download the comment data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadCommentDataList(
            s_commentDataList, &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadCommentDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }

    // Display the information for the downloaded comment data.
    for (int i=0; i<downloadListSize; ++i)
    {
        PrintDownloadedCommentData(s_commentDataList[i]);
    }
    
    NN_LOG("nn::olv::DownloadCommentDataList success.\n");
}

void DemoMain()
{
    char postId[nn::olv::POST_ID_BUFF_LENGTH];

    if (strlen(s_TargetPostId) > 0)
    {
        // Use the configured post ID.
        strncpy(postId, s_TargetPostId, nn::olv::POST_ID_BUFF_LENGTH);
    }
    else
    {
        // Download the post data and get the post ID.
        if (!GetPostIdFromDownloadData(postId, nn::olv::POST_ID_BUFF_LENGTH))
        {
            NN_LOG("ERROR: GetPostIdFromDownloadData failed.\n");
            return;
        }
    }
    
    // Download the comment data.
    DownloadCommentDataListSample(postId);
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
