﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_DownloadCommunityDataList_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of downloading community data.
 It receives DOWNLOAD_MAX_NUM sets of community data and displays the results.
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for the community data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadedCommunityData s_communityDataList[DOWNLOAD_MAX_NUM];
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "DownloadCommunityDataList demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

// The process of downloading community data.
void DemoMain()
{
    nn::olv::Result result;

    // Parameter when downloading community data.
    nn::olv::DownloadCommunityDataListParam downloadParam;
    
    // Set the maximum number to download.
    downloadParam.SetCommunityDataMaxNum(DOWNLOAD_MAX_NUM);
     
    u32 flags = 0;

    // Set the type of communities you want to download.
    // Set only one of the following: FLAG_FILTER_BY_SELF / FLAG_FILTER_BY_FAVORITE / FLAG_FILTER_BY_OFFICIAL.
    // The API call fails if there are multiple settings or if nothing has been set.
    //flags |= nn::olv::DownloadCommunityDataListParam::FLAG_FILTER_BY_SELF; // Get only communities you yourself have created.
    //flags |= nn::olv::DownloadCommunityDataListParam::FLAG_FILTER_BY_FAVORITE; // Get only favorite communities.
    flags |= nn::olv::DownloadCommunityDataListParam::FLAG_FILTER_BY_OFFICIAL; // Get only official communities.
    

    // The following options relate to data downloaded at the same time. Set them as deemed necessary.
    // If you want the downloaded data to include the Mii character data for the creator of the community, set the following flag.
    //flags |= nn::olv::DownloadCommunityDataListParam::FLAG_WITH_MII_DATA;

    // If you want the downloaded data to include the icon data for the community, set the following flag.
    flags |= nn::olv::DownloadCommunityDataListParam::FLAG_WITH_ICON_DATA;
    downloadParam.SetFlags(flags);
    
    // If you only want to download the data for a specific community, set the ID for that community.
    // 
    // (If you have set a community ID, the only flags you can set with SetFlags are FLAG_WITH_MII_DATA and FLAG_WITH_ICON_DATA.)
    {
        // Specify indices to set multiple community IDs.
        // You can set up to a maximum of nn::olv::COMMUNITY_ID_MAX_NUM community IDs.
        //downloadParam.SetCommunityId(0L, 0);
    }

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadCommunityDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);
 
    // Download the community data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadCommunityDataList(
            s_communityDataList, &downloadListSize,
            &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadCommunityDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
    
    // Display the data content of the downloaded communities.
    NN_LOG("nn::olv::DownloadCommunityDataList success.\n");
    NN_LOG("download list size %u\n", downloadListSize);
    
    for (s32 i = 0; i < downloadListSize; ++i)
    {
        NN_LOG("----- CommunityData [%d] -----\n", i);
        
        if (s_communityDataList[i].TestFlags(nn::olv::DownloadedCommunityData::FLAG_WITH_TITLE_TEXT))
        {
            const u32 titleTextBuffLength = nn::olv::TITLE_TEXT_BUFF_LENGTH;
            wchar_t titleText[titleTextBuffLength];
    
            s_communityDataList[i].GetTitleText(titleText, titleTextBuffLength);
            NN_LOG("TitleText: ");
            OLVDemo_PrintWideStr(titleText);
        }
        
        if (s_communityDataList[i].TestFlags(nn::olv::DownloadedCommunityData::FLAG_WITH_DESCRIPTION_TEXT))
        {
            const u32 descriptionTextBuffLength = nn::olv::DESCRIPTION_TEXT_BUFF_LENGTH;
            wchar_t descriptionText[descriptionTextBuffLength];
    
            s_communityDataList[i].GetDescriptionText(descriptionText, descriptionTextBuffLength);
            NN_LOG("DescriptionText: ");
            OLVDemo_PrintWideStr(descriptionText);
        }
        
        NN_LOG("CommunityId: %u\n", s_communityDataList[i].GetCommunityId());
        NN_LOG("OwnerPid: %u\n", s_communityDataList[i].GetOwnerPid());
        
        // Get the application data.
        if (s_communityDataList[i].TestFlags(nn::olv::DownloadedCommunityData::FLAG_WITH_APP_DATA))
        {
            const u32 bufferSize = s_communityDataList[i].GetAppDataSize();
            OLVDemo_MemoryManager workMemory;
            if ((workMemory.Allocate(bufferSize)))
            {
                u32 dataSize = 0;
                s_communityDataList[i].GetAppData(workMemory.buf, &dataSize, bufferSize);
                NN_LOG("AppData: ");
                OLVDemo_PrintBinary(workMemory.buf);
                NN_LOG("app data size: %u\n", dataSize);
            }
        }
        
        // Get icon data.
        if (s_communityDataList[i].TestFlags(nn::olv::DownloadedCommunityData::FLAG_WITH_ICON_DATA))
        {
            OLVDemo_MemoryManager workMemory;
            if ((workMemory.Allocate(nn::olv::ICON_DATA_BUFF_LENGTH)))
            {
                u32 dataSize = 0;
                result = s_communityDataList[i].GetIconData(workMemory.buf, &dataSize, nn::olv::ICON_DATA_BUFF_LENGTH);
                if (result.IsSuccess())
                {
                    NN_LOG("IconData: ");
                    OLVDemo_PrintBinary(workMemory.buf);
                    NN_LOG("icon data size: %u\n", dataSize);
                }
            }
        }
        
        // Gets the Mii character data of the community's creator.
        if (s_communityDataList[i].TestFlags(nn::olv::DownloadedCommunityData::FLAG_WITH_MII_DATA))
        {
            nn::mii::StoreData miiData;

            s_communityDataList[i].GetOwnerMiiData(&miiData);
            NN_LOG("OwnerMiiData:");
            OLVDemo_PrintBinary((u8*)&miiData);
        }
        
        // Gets the Mii character nickname of the community's creator.
        wchar_t ownerMiiNickNameBuff[nn::olv::MII_NICKNAME_BUFF_LENGTH];
        s_communityDataList[i].GetOwnerMiiNickname(ownerMiiNickNameBuff, nn::olv::MII_NICKNAME_BUFF_LENGTH);
        NN_LOG("OwnerMiiNickname: ");
        OLVDemo_PrintWideStr(ownerMiiNickNameBuff);
    }
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
