﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_DownloadPostDataList_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of downloading post data.
 It receives the post data for DOWNLOAD_MAX_NUM of posts and displays the content.
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for the post data and topic data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadedPostData s_postDataList[DOWNLOAD_MAX_NUM];
    nn::olv::DownloadedTopicData s_topicData;
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "DownloadPostDataList demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

// The process of downloading post data.
void DemoMain()
{
    nn::olv::Result result;
    
    // Parameter when downloading post data.
    nn::olv::DownloadPostDataListParam downloadParam;
    
    // Set the maximum number to download.
    downloadParam.SetPostDataMaxNum(DOWNLOAD_MAX_NUM);

    // Set a search key if you want to download posts that contain that particular search key.
    // UTF-16 LE encoding of "key0."
    const wchar_t key0[] = { 0x006B, 0x0065, 0x0079, 0x0030, 0x0000 };
    downloadParam.SetSearchKey(key0);
        
    u32 flags = 0;    
    // Set the flags for the get targets.
    // If no flags are set, then posts will be retrieved from all possible targets (friends, followers, your own posts, and posts other users).
    // To filter for specific targets, set one or more of the following flags.
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_FRIEND;
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_FOLLOW;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_SELF;    
    // Set the data body type.
    // If this flag is not set, then the data will be retrieved with no distinction between text and handwritten memos.
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_BODY_TEXT;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_BODY_MEMO;    
    // Downloads post data including Mii character data.
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_WITH_MII_DATA;    
    // Download only a single post from each poster.
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_DISTINCT_PID;    
    // Download posts, including whether you yourself have set a Yeah flag for the post.
    //flags |= nn::olv::DownloadPostDataListParam::FLAG_WITH_EMPATHY_ADDED;    
    //flags |= nn::olv::hidden::DownloadSystemPostDataListParam::FLAG_SORT_BY_POPULAR;
    //flags |= nn::olv::hidden::DownloadSystemPostDataListParam::FLAG_WITH_MII_DATA;
    // Download post data, including those with spoilers.
    flags |= nn::olv::DownloadPostDataListParam::FLAG_WITH_SPOILER;

    downloadParam.SetFlags(flags);
    
    // If you only want to download certain post data, set the post IDs.
    // When setting a post ID, the only options that you can set are FLAG_WITH_MII_DATA and FLAG_WITH_EMPATHY_ADDED.
    // 
    // You can set up to a maximum of nn::olv::POST_ID_MAX_NUM post IDs.
    //downloadParam.SetPostId("", 0);
    
    // Set a principal ID if you only want to download posts from a certain user.
    // This setting cannot be used together with any of the following flags.
    // FLAG_FILTER_BY_FRIEND, FLAG_FILTER_BY_FOLLOW, FLAG_FILTER_BY_SELF
    //downloadParam.SetSearchPid(0L);

    // If you want to download post data from anything other than the default general communities, set the IDs for those other communities.
    // 
    //downloadParam.SetCommunityId(99);
    
    // Set a language ID if you want to filter for only a certain language.
    //downloadParam.SetLanguageId(nn::olv::LANGUAGE_ID_ALL);
    
    // Download based on specified date and time of post.
    // Set the date and time in the format of nn::fnd::DateTime.
    //nn::fnd::DateTime date (2013, 1, 1, 0, 0, 0);
    //downloadParam.SetPostDate(date);

    // Set the maximum number of characters of text data.
    downloadParam.SetBodyTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadPostDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the post data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadPostDataList(
            &s_topicData, s_postDataList,
            &downloadListSize, &downloadParam);
    
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadPostDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
    
    // Display the downloaded post data.
    NN_LOG("nn::olv::DownloadPostDataList success.\n");
    NN_LOG("download list size %u\n", downloadListSize);
    
    for (s32 i = 0; i < downloadListSize; ++i)
    {
        NN_LOG("----- PostData [%d] -----\n", i);
        
        OLVDemo_PrintDate(s_postDataList[i].GetPostDate());
        
        // Get the text data.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_BODY_TEXT))
        {
            const u32 bodyTextBuffLength = nn::olv::BODY_TEXT_BUFF_LENGTH;
            wchar_t bodyText[bodyTextBuffLength];

            s_postDataList[i].GetBodyText(bodyText, bodyTextBuffLength);
            NN_LOG("BodyText: ");
            OLVDemo_PrintWideStr(bodyText);
        }
        
        // Get the handwritten memo data.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_BODY_MEMO))
        {
            OLVDemo_MemoryManager workMemory;
            if ((workMemory.Allocate(nn::olv::BODY_MEMO_BUFF_LENGTH)))
            {
                u32 dataSize = 0;
                s_postDataList[i].GetBodyMemo(workMemory.buf, &dataSize, nn::olv::BODY_MEMO_BUFF_LENGTH);
                NN_LOG("BodyMemo: ");
                OLVDemo_PrintBinary(workMemory.buf);
                NN_LOG("memo data size: %u\n", dataSize);
            }
        }
        
        NN_LOG("UserPid: %u\n", s_postDataList[i].GetUserPid());
        char postIdBuff[nn::olv::POST_ID_BUFF_LENGTH];
        s_postDataList[i].GetPostId(postIdBuff, nn::olv::POST_ID_BUFF_LENGTH);
        NN_LOG("PostId: %s\n", postIdBuff);
        NN_LOG("CommunityId: %u\n", s_postDataList[i].GetCommunityId());
        NN_LOG("Feeling: %d\n", s_postDataList[i].GetFeeling());
        NN_LOG("EmpathyCount: %d\n", s_postDataList[i].GetEmpathyCount());
        NN_LOG("CommentCount: %d\n", s_postDataList[i].GetCommentCount());
        NN_LOG("RegionId: %u\n", s_postDataList[i].GetRegionId());
        NN_LOG("PlatformId: %u\n", s_postDataList[i].GetPlatformId());
        NN_LOG("LanguageId: %u\n", s_postDataList[i].GetLanguageId());
        NN_LOG("CountryId: %u\n", s_postDataList[i].GetCountryId());
        NN_LOG("EmpathyAdded: %s\n", 
            s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_EMPATHY_ADDED) ? 
            "true" : "false");
        NN_LOG("IsSpoiler: %s\n", 
            s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_SPOILER) ? 
            "true" : "false");
        NN_LOG("FreeFormat: %s\n", 
            s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_FREE_FORMAT) ? 
            "true" : "false");

        // Get the external URL.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_EXTERNAL_URL))
        {
            char externalUrlBuff[nn::olv::EXTERNAL_URL_BUFF_LENGTH];
            s_postDataList[i].GetExternalUrl(externalUrlBuff, nn::olv::EXTERNAL_URL_BUFF_LENGTH);
            NN_LOG("ExternalUrl: %s\n", externalUrlBuff);
        }

        // Get the Mii character data.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_MII_DATA))
        {
            nn::mii::StoreData miiData;

            s_postDataList[i].GetMiiData(&miiData);
            NN_LOG("MiiData:");
            OLVDemo_PrintBinary((u8*)&miiData);
        }
        
        // Get the application data.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_APP_DATA))
        {
            const u32 bufferSize = s_postDataList[i].GetAppDataSize();
            OLVDemo_MemoryManager workMemory;
            if ((workMemory.Allocate(bufferSize)))
            {
                u32 dataSize = 0;
                s_postDataList[i].GetAppData(workMemory.buf, &dataSize, bufferSize);
                NN_LOG("AppData: ");
                OLVDemo_PrintBinary(workMemory.buf);
                NN_LOG("app data size: %u\n", dataSize);
            }
        }
        
        // Download any images attached to the posts.
        if (s_postDataList[i].TestFlags(nn::olv::DownloadedPostData::FLAG_WITH_EXTERNAL_IMAGE_DATA))
        {
            const u32 bufferSize = s_postDataList[i].GetExternalImageDataSize();
            OLVDemo_MemoryManager workMemory;
            if ((workMemory.Allocate(bufferSize)))
            { 
                NN_LOG("ExternalImageData receiving...\n");
                u32 downloadSize = 0;
                // Download the attached image data (Internet communication takes place).
                result = s_postDataList[i].DownloadExternalImageData(workMemory.buf, &downloadSize, bufferSize);
                if (result.IsSuccess())
                {
                    NN_LOG("ExternalImageData: ");
                    OLVDemo_PrintBinary(workMemory.buf);
                    NN_LOG("download size/max: %u/%u\n", downloadSize, bufferSize);
                }
            }
        }
    }
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
