﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_DownloadUserDataList_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of downloading comment data.

 * Information on users who Yeah'ed post data
 * Information on users who favorited a community
 * Information on users who are friends with another user or are following another user
 
 All of the above is downloaded, and the results are displayed.
 
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include <nn/act/act_Api.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for the post data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadedPostData s_postDataList[DOWNLOAD_MAX_NUM];
    nn::olv::DownloadedTopicData s_topicData;

    // Container for the user data.
    const u32 USERDATA_DOWNLOAD_MAX_NUM = 20;
    nn::olv::DownloadedUserData s_userDataList[USERDATA_DOWNLOAD_MAX_NUM];
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "DownloadUserDataList demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

bool GetPostIdFromDownloadData(u32 &userPid, char *postIdBuffer, const u32 postIdBufferSize)
{
    nn::olv::Result result;

    // Download the post data and get the IDs of posts that have Yeahs.
    nn::olv::DownloadPostDataListParam downloadParam;
    
    downloadParam.SetPostDataMaxNum(DOWNLOAD_MAX_NUM);
 
    // Set the flag
    u32 flags = 0;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_BODY_TEXT;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_WITH_EMPATHY_ADDED;
    downloadParam.SetFlags(flags);
 
    u32 downloadListSize = 0;
       
    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadPostDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    result = nn::olv::DownloadPostDataList(
            &s_topicData, s_postDataList,
            &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadPostDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }

    // Search for post data within one or more Yeahs, and copy the post and principal IDs.
    for (s32 i = 0; i < downloadListSize; ++i)
    {
        if (s_postDataList[i].GetEmpathyCount() > 0)
        {
            char postIdTmpBuff[nn::olv::POST_ID_BUFF_LENGTH];
            s_postDataList[i].GetPostId(postIdTmpBuff, nn::olv::POST_ID_BUFF_LENGTH);
            strncpy(postIdBuffer, postIdTmpBuff, postIdBufferSize);
            userPid = s_postDataList[i].GetUserPid();
            return true;
        }
    }

    // Result in an error since the target post data could not be found.
    NN_LOG("ERROR: GetPostIdFromDownloadData download success. but target post not found.\n");

    return false;
}

// Output the information for the obtained user data.
void PrintUserDataInfo(const nn::olv::DownloadedUserData* userData, u32 downloadListSize)
{
    NN_LOG("nn::olv::DownloadUserDataList success.\n");
    NN_LOG("download list size %u\n", downloadListSize);

    for (int i = 0; i < downloadListSize; ++i)
    {
        NN_LOG("----- UserData [%d] -----\n", i);
        
        // Show the principal IDs of the users.
        NN_LOG("UserPid: %u\n", userData[i].GetUserPid());
        
        // Show the Mii character nicknames.
        wchar_t miiNickname[nn::olv::MII_NICKNAME_BUFF_LENGTH];
        NN_LOG("MiiNickname: ");
        userData[i].GetMiiNickname(miiNickname, nn::olv::MII_NICKNAME_BUFF_LENGTH);
        OLVDemo_PrintWideStr(miiNickname);
        
        // Get the Mii character data.
        if (userData[i].TestFlags(nn::olv::DownloadedUserData::FLAG_WITH_MII_DATA))
        {
            nn::mii::StoreData miiData;

            userData[i].GetMiiData(&miiData);
            NN_LOG("MiiData: ");
            OLVDemo_PrintBinary((u8*)&miiData);
        }
    }
}

// Get information on users who Yeah'ed post data.
void DownloadUserData_EmpathyToPost(const char* postId)
{
    nn::olv::Result result;
    
    nn::olv::DownloadUserDataListParam downloadParam;

    // Specify the post ID.
    downloadParam.SetPostId(postId);
     
    // The get flag settings.
    u32 flags = 0;    
    // Set the flags for the get targets.
    // Set FLAG_EMPATHY_TO_POST.
    flags |= nn::olv::DownloadUserDataListParam::FLAG_EMPATHY_TO_POST;
    // Gets user data, including Mii character data.
    // You do not need to set this flag if you do not need the Mii character data.
    flags |= nn::olv::DownloadUserDataListParam::FLAG_WITH_MII_DATA;

    // Set the flags.
    downloadParam.SetFlags(flags);

    // Set the maximum number to get.
    downloadParam.SetUserDataMaxNum(USERDATA_DOWNLOAD_MAX_NUM);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadUserDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the user data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadUserDataList(s_userDataList, &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadUserDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
   
    // Output the information
    PrintUserDataInfo(s_userDataList, downloadListSize);
 }

// Get information for users who favorited communities.
void DownloadUserData_FavoriteToCommunity(u32 communityId)
{
    nn::olv::Result result;
    
    nn::olv::DownloadUserDataListParam downloadParam;

    // Specify the community ID.
    downloadParam.SetCommunityId(communityId);
     
    // The get flag settings.
    u32 flags = 0;    
    // Set the flags for the get targets.
    // Set FLAG_EMPATHY_TO_POST.
    flags |= nn::olv::DownloadUserDataListParam::FLAG_FAVORITE_TO_COMMUNITY;
    // Gets user data, including Mii character data.
    // You do not need to set this flag if you do not need the Mii character data.
    flags |= nn::olv::DownloadUserDataListParam::FLAG_WITH_MII_DATA;

    // Set the flags.
    downloadParam.SetFlags(flags);

    // Set the maximum number to get.
    downloadParam.SetUserDataMaxNum(USERDATA_DOWNLOAD_MAX_NUM);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadUserDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the user data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadUserDataList(s_userDataList, &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadUserDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
   
    // Output the information
    PrintUserDataInfo(s_userDataList, downloadListSize);
}

// Get information on users who are friends with another user or are following another user.
void DownloadUserData_FollowToUser(u32 userPid)
{
    nn::olv::Result result;
    
    nn::olv::DownloadUserDataListParam downloadParam;

    // Specify the principal ID.
    downloadParam.SetUserPid(userPid);
     
    // The get flag settings.
    u32 flags = 0;    
    // Set the flags for the get targets.
    // To get friends, set FLAG_FRIEND_TO_USER. To get followers, set FLAG_FOLLOW_TO_USER.
    // 
    // (You cannot set both flags at the same time.)
    flags |= nn::olv::DownloadUserDataListParam::FLAG_FOLLOW_TO_USER;
    //flags |= nn::olv::DownloadUserDataListParam::FLAG_FRIEND_TO_USER;
    // Gets user data, including Mii character data.
    // You do not need to set this flag if you do not need the Mii character data.
    flags |= nn::olv::DownloadUserDataListParam::FLAG_WITH_MII_DATA;

    // Set the flags.
    downloadParam.SetFlags(flags);

    // Set the maximum number to get.
    downloadParam.SetUserDataMaxNum(USERDATA_DOWNLOAD_MAX_NUM);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadUserDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the user data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadUserDataList(s_userDataList, &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadUserDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
   
    // Output the information
    PrintUserDataInfo(s_userDataList, downloadListSize);
}

// The process of uploading post data.
void DemoMain()
{
    char postId[nn::olv::POST_ID_BUFF_LENGTH];
    u32 userPid;

    // Get the principal and post IDs from post data within one or more Yeahs.
    //
    // NOTE: The demo will fail when unable to get any post data with Yeahs.
    // In that case, make adjustments, such as increasing the value of DOWNLOAD_MAX_NUM, or making sure that posts have been Yeahed in advance, to improve the download process.
    // 
    //
    if (!GetPostIdFromDownloadData(userPid, postId, nn::olv::POST_ID_BUFF_LENGTH))
    {
        NN_LOG("ERROR: GetPostIdFromDownloadData failed.\n");
        return;
    }
    
    // Gets the user information.
    DownloadUserData_EmpathyToPost(postId);
    DownloadUserData_FavoriteToCommunity(0);

    // Note:
    // When getting information for users who are friends with another user or are following another user, if the target user has limited privacy settings on his or her friends and/or follows, the get will fail and an error will result (for more information see the Error/EULA display applet shown in the lower screen).
    // 
    // 
    // To avoid this, you need to have the user set his or Miiverse profile settings to not restrict privacy.
    // 
    // This demo attempts to work using the principal ID for the post data that has been Yeahed, but when that fails for an above error, retry using the principal ID of a friend that has a userPid that has been appropriately configured for privacy.
    // 
    // 

    // Directly specify the principal ID.
    // userPid = 1234567890;
    DownloadUserData_FollowToUser(userPid);
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
