﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_StartPortalApp_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of starting the Miiverse application.

 You move the cursor up/down using the +Control Pad and press the A Button to start the Miiverse application in the selected mode .
 
 
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for the OLV restore process
    nn::olv::ResumedData s_resumedData;
    nn::olv::ResumeParam s_resumeParam;

    // Buffer that takes system arguments when launched from the Miiverse application
    u8 s_systemArgs[NN_APPLET_PARAMETER_MAX];

    // Cursor position.
    s32 s_cursorPos = 0;

    // Text string to be drawn
    const char *s_menuStrings[] = {
        "StartPortalApp default",
        "StartPortalApp with community",
        "StartPortalApp with postId",
        "StartPortalApp with user",
    };
}


namespace sample { namespace olv {

void StartPortalApp(u32 mode)
{
    nn::olv::StartPortalAppParam startParam;
    nn::olv::Result result;

    switch (mode)
    {
        case nn::olv::StartPortalAppParam::MODE_NONE:
            {   // This is the default. The application starts with nothing specified.
            }
            break;

        case nn::olv::StartPortalAppParam::MODE_START_COMMUNITY:
            {
                // Specify the ID of the community to start with.
                u32 communityId = 0;
                startParam.SetMode(mode);
                startParam.SetCommunityId(communityId);
            }
            break;

        case nn::olv::StartPortalAppParam::MODE_START_POST:
            {
                // Specify the ID of the post to start with.
                const char postId[nn::olv::POST_ID_BUFF_LENGTH] = "POSTID";
                startParam.SetMode(mode);
                startParam.SetPostId(postId);
            }
            break;

        case nn::olv::StartPortalAppParam::MODE_START_USER:
            {
                // Specify the principal ID to start with.
                u32 principalId= 123456789;
                startParam.SetMode(mode);
                startParam.SetUserPid(principalId);
            }
            break;
    }

    // Get and set the work size.
    u32 workSize = startParam.GetWorkSize();
    NN_LOG("nn::olv::StartPortalApp - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    // NOTES
    // - Note that you must not use device memory for the buffer passed to the nn::olv::StartPortalAppParam::SetWork function.
    //   Also, do not destroy this buffer until the nn::olv::Resume function is called.
    startParam.SetWork(workMemory.buf, workSize);

    result = nn::olv::StartPortalApp(&startParam);
    if (result.IsSuccess())
    {
        AppletId senderId = 0;
        s32 systemArgsSize = 0;

        // The Miiverse application starts, interrupting the application.
        // Wait for the application to start.
        nn::applet::WaitForStarting(&senderId, s_systemArgs, NN_APPLET_PARAMETER_MAX, &systemArgsSize);

        // Call the resume process of the OLV library.
        s_resumedData.Reset();
        s_resumeParam.Reset();
        if (senderId == nn::olv::PORTAL_APP_APPLET_ID)
        {
            // Specifies when system arguments are passed from the Miiverse application.
            s_resumeParam.SetSystemArgs(s_systemArgs, (u32)systemArgsSize);
        }
        result = nn::olv::Resume(&s_resumedData, &s_resumeParam);
        if (result.IsFailure())
        {
            NN_LOG("nn::olv::Resume failed.\n");
            OLVDemo_ErrorHandling(result);
        }

        // Termination request.
        if (nn::applet::CTR::IsExpectedToCloseApplication())
        {
            s_isEndDemo = true;
            return;
        }

        // Recover the GPU register settings
        nngxUpdateState(NN_GX_STATE_ALL);
        nngxValidateState(NN_GX_STATE_ALL,GL_TRUE);
    }
    else
    {
        NN_LOG("nn::olv::StartPortalApp failed.\n");
        OLVDemo_ErrorHandling(result);
    }
}

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    // Upper screen.
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "StartPortalApp demo");

    renderSystem.SwapBuffers();

    // Lower screen.
    renderSystem.SetRenderTarget(NN_GX_DISPLAY1);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    if (nn::olv::IsInitialized())
    {
        // Draw menu and cursor.
        int i = 0;
        for (; i<=nn::olv::StartPortalAppParam::MODE_START_USER; ++i)
        {
            if (i == s_cursorPos)
            {
                renderSystem.DrawText(0, LINE_HEIGHT * i, ">");
            }
            renderSystem.DrawText(LINE_WIDTH, LINE_HEIGHT * i, s_menuStrings[i]);
        }
        i++;
        renderSystem.DrawText(LINE_WIDTH, LINE_HEIGHT * i++, "press B to end demo");
    }

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (nn::olv::IsInitialized())
    {
        if (!padReader.ReadLatest(&padStatus))
        {
            nn::dbg::detail::TPrintf("Getting pad status failed.\n");
        }

        if (padStatus.trigger & nn::hid::BUTTON_UP)
        {
            if (s_cursorPos == nn::olv::StartPortalAppParam::MODE_NONE)
            {
                s_cursorPos = nn::olv::StartPortalAppParam::MODE_START_USER;
            }
            else
            {
                --s_cursorPos;
            }
        }
        else if (padStatus.trigger & nn::hid::BUTTON_DOWN)
        {
            if (s_cursorPos == nn::olv::StartPortalAppParam::MODE_START_USER)
            {
                s_cursorPos = nn::olv::StartPortalAppParam::MODE_NONE;
            }
            else
            {
                ++s_cursorPos;
            }
        }
        else if (padStatus.trigger & nn::hid::BUTTON_A)
        {   // Start the Miiverse application in the selected mode.
            StartPortalApp(s_cursorPos);
        }
        else if (padStatus.trigger & nn::hid::BUTTON_B)
        {   // End the demo.
            s_isEndDemo = true;
        }
    }
}

void DemoMain()
{
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();

    if (!s_isEndDemo)
    {
        ProceedDisplay(renderSystem);

        if (nn::olv::IsConnected())
        {
            DemoMain();
        }

        nngxWaitVSync(NN_GX_DISPLAY_BOTH);
    }

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    nn::applet::AppletId senderId = 0;
    s32 systemArgsSize = 0;

    // Takes system arguments when launched from the Miiverse application
    nn::applet::GetInitialParam(&senderId, s_systemArgs, NN_APPLET_PARAMETER_MAX, &systemArgsSize);

    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    // Call the resume process of the OLV library.
    s_resumedData.Reset();
    s_resumeParam.Reset();
    if (senderId == nn::olv::PORTAL_APP_APPLET_ID)
    {
        // Specifies when system arguments are passed from the Miiverse application.
        s_resumeParam.SetSystemArgs(s_systemArgs, (u32)systemArgsSize);
    }
    result = nn::olv::Resume(&s_resumedData, &s_resumeParam);
    if (result.IsFailure())
    {
       OLVDemo_ErrorHandling(result);
       NN_LOG("nn::olv::Resume failed.\n");
       return false;
    }

    // Because the Internet connection for this demo occurs within the Miiverse application, there is no acquisition of service tokens or OLV connection within the demo.
    // 
    OLVDemo_SetIsConnect(false);

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    // Nothing is done here because no connection process is conducted in this demo.
    // (This function is not called.)
    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
