﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_UploadCommunityData_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of processes related to the uploading of community data.
 It performs operations to create, update, and delete community data.
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Parameter for downloading community data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadCommunityDataListParam s_downloadParam;
    nn::olv::DownloadedCommunityData s_downloadedData[DOWNLOAD_MAX_NUM];

    // Container for uploaded data.
    nn::olv::UploadedCommunityData s_uploadedData;

    // Where to get icon image.
    u8 s_iconBuffer[nn::olv::ICON_DATA_BUFF_LENGTH];
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "UploadCommunityData demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

// Delete all community data created by you yourself.
// There is a limit to how many communities can be created, so  call this function as required.
bool DeleteAllCommunitiesSample()
{
    nn::olv::Result result;
    u32 downloadedDataNum;

    s_downloadParam.Reset();
    s_downloadParam.SetFlags(nn::olv::DownloadCommunityDataListParam::FLAG_FILTER_BY_SELF);
    s_downloadParam.SetCommunityDataMaxNum(DOWNLOAD_MAX_NUM);

    // Get and set the work size.
    u32 workSize = s_downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadCommunityData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    s_downloadParam.SetWork(workMemory.buf, workSize);

    result = nn::olv::DownloadCommunityDataList(s_downloadedData, &downloadedDataNum, &s_downloadParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("DeleteAllCommunitiesSample failed.\n");
        return false;
    }

    NN_LOG("downloadedDataNum=%d\n", downloadedDataNum);
    if (downloadedDataNum == 0)
    {   // There are no community data to delete, so just return.
        return true;
    }

    // Delete community data.
    for (int i=0; i<downloadedDataNum; i++)
    {
        nn::olv::UploadCommunityDataParam uploadParam;

        // Get the community ID.
        u32 communityId = s_downloadedData[i].GetCommunityId();

        NN_LOG("deleting old community %u\n", communityId);
        uploadParam.SetFlags(nn::olv::UploadCommunityDataParam::FLAG_DELETE);
        uploadParam.SetCommunityId(communityId);
        u32 uploadWorkSize = uploadParam.GetWorkSize();
        NN_LOG("nn::olv::DownloadCommunityData - work size:%d\n", workSize);
        OLVDemo_MemoryManager uploadWorkMemory;
        if ((uploadWorkMemory.Allocate(uploadWorkSize)) == NULL)
        {
            NN_LOG("heap allocate failed. size:%d\n", uploadWorkSize);
            return false;
        }
        uploadParam.SetWork(uploadWorkMemory.buf, uploadWorkSize);

        result = nn::olv::UploadCommunityData(NULL, &uploadParam);
        if (result.IsFailure())
        {
            OLVDemo_ErrorHandling(result);
            return false;
        }
    }

    return true;
}

// Create communities.
//
// CAUTION
// There is a limit to the number of communities that a single user can create for any single title.
// An error occurs if the maximum number has been created already.
// For details, see the error code list.
//
// To create new community data in this data, you will first need to delete some community data.
// 
// This demo includes DeleteAllCommunitiesSample, which you can execute ahead of time if you want.
//
bool CreateCommunitySample(u32* communityId)
{
    nn::olv::Result result;
    
    // Parameter for creating community.
    nn::olv::UploadCommunityDataParam uploadParam;

    // Set to 0 to create a community. (The default is 0, so this does not need to be set.)
    uploadParam.SetCommunityId(0);

    // Set the name for the community.
    // UTF-16 LE encoding of "test community" (in Japanese).
    const wchar_t title[] = { 
        0x30C6, 0x30B9, 0x30C8, 0x30B3, 0x30DF, 
        0x30E5, 0x30CB, 0x30C6, 0x30A3, 0x0000
    };
    uploadParam.SetTitleText(title);
    
    // Set a description of the community data.
    // UTF-16 LE encoding of "Test community created with demo" (in Japanese).
    const wchar_t description[] = { 
        0x30C7, 0x30E2, 0x3067, 0x4F5C, 0x6210, 0x3057, 0x305F, 
        0x30C6, 0x30B9, 0x30C8, 0x30B3, 0x30DF, 0x30E5, 0x30CB,
        0x30C6, 0x30A3, 0x3067, 0x3059, 0x3002, 0x0000 
    };
    uploadParam.SetDescriptionText(description);
    
    // Specifies the maximum number of strings that can be posted from the Miiverse application.
    uploadParam.SetPostTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);
    
    // Specifies the maximum number of strings that can be commented from the Miiverse application.
    uploadParam.SetCommentTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);

    // Set the application data.
    const u8 userData[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@";
    uploadParam.SetAppData(userData, sizeof(userData));
    
    // Set the icon data.
    OLVDemo_FileManager imageData;
    if (!imageData.Read("icon001.tga"))
    {
        NN_LOG("read icon data failed:%s\n", imageData.GetFileName());
        return false;
    }
    uploadParam.SetIconData(static_cast<u8*>(imageData.data), imageData.size);
   
    // Get and set the work size.
    u32 workSize = uploadParam.GetWorkSize();
    NN_LOG("nn::olv::UploadCommunityData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    uploadParam.SetWork(workMemory.buf, workSize);

    // Create community.
    // Set container for the result (if not necessary, set to NULL) at same time as the upload parameter, and call the API.
    result = nn::olv::UploadCommunityData(&s_uploadedData, &uploadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadCommunityData failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }

    NN_LOG("nn::olv::UploadCommunityData - create - success.\n");

    // If the upload is successful, the result after uploading gets stored.
    // Get the community ID for this newly created community data.
    NN_LOG("communityId: %u\n", s_uploadedData.GetCommunityId());
    *communityId = s_uploadedData.GetCommunityId();
       
    return true;
}

// Update community.
bool UpdateCommunitySample(const u32 communityId)
{
    nn::olv::Result result;
    
    // Parameter for updating community.
    nn::olv::UploadCommunityDataParam uploadParam;

    // Set the ID of the community to update.
    uploadParam.SetCommunityId(communityId);
    
    // Update the description for the community.
    // Set values to parameters you want to update using the same method used to create a community.
    // UTF-16 LE encoding of "Test community has been updated" (in Japanese).
    const wchar_t description[] = { 
        0x30C6, 0x30B9, 0x30C8, 0x30B3, 0x30DF, 0x30E5, 0x30CB, 0x30C6, 0x30A3, 
        0x3092, 0x66F4, 0x65B0, 0x3057, 0x307E, 0x3057, 0x305F, 0x3002, 0x0000
    };
    uploadParam.SetDescriptionText(description);

    // Specifies the maximum number of strings that can be posted from the Miiverse application.
    uploadParam.SetPostTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);
    
    // Specifies the maximum number of strings that can be commented from the Miiverse application.
    uploadParam.SetCommentTextMaxLength(nn::olv::BODY_TEXT_MAX_LENGTH);

    // Set the icon data.
    OLVDemo_FileManager imageData;
    if (!imageData.Read("icon001.tga"))
    {
        NN_LOG("read icon data failed:%s\n", imageData.GetFileName());
        return false;
    }
    uploadParam.SetIconData(static_cast<u8*>(imageData.data), imageData.size);

    // Get and set the work size.
    u32 workSize = uploadParam.GetWorkSize();
    NN_LOG("nn::olv::UploadCommunityData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    uploadParam.SetWork(workMemory.buf, workSize);

    // Update community.
    result = nn::olv::UploadCommunityData(&s_uploadedData, &uploadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadCommunityData failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }
 
    NN_LOG("nn::olv::UploadCommunityData - update - success.\n");

    // Output the icon data of the updated community data.
    u32 iconSize;
    s_uploadedData.GetIconData(s_iconBuffer, &iconSize, nn::olv::ICON_DATA_BUFF_LENGTH);
    NN_LOG("icon size:%d icon data:\n", iconSize);
    OLVDemo_PrintBinary(s_iconBuffer, 200);
    
    return true;
}

// Delete community.
bool DeleteCommunitySample(const u32 communityId)
{
    nn::olv::Result result;
    
    // Parameter for deleting community.
    nn::olv::UploadCommunityDataParam uploadParam;

    // Set the ID of the community to delete.
    uploadParam.SetCommunityId(communityId);
        
    // Set the flag to indicate the deletion of the community.
    uploadParam.SetFlags(nn::olv::UploadCommunityDataParam::FLAG_DELETE);

    // Get and set the work size.
    u32 workSize = uploadParam.GetWorkSize();
    NN_LOG("nn::olv::UploadCommunityData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workMemory;
    if ((workMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    uploadParam.SetWork(workMemory.buf, workSize);

    // Delete a community.
    result = nn::olv::UploadCommunityData(NULL, &uploadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadCommunityData failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }
 
    NN_LOG("nn::olv::UploadCommunityData - delete - success.\n");
    
    return true;
}

// The process of uploading community data.
void DemoMain()
{
    // Get the ID for the created community.
    u32 communityId = 0;
    
    // Delete all communities created by you yourself.
    // (Call this function as needed.)
    //DeleteAllCommunitiesSample();

    // Create the community.
    if (!CreateCommunitySample(&communityId))
    {
        return;
    }

    // Update (change the content) of a created community.
    if (!UpdateCommunitySample(communityId))
    {
        return;
    }
   
    // Delete a created community.
    if (!DeleteCommunitySample(communityId))
    {
        return;
    }
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
