﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_UploadEmpathyToPostData_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of making a Yeah on a post or removing a Yeah from a post.

 Demo process flow:
 * Downloads post data for DOWNLOAD_MAX_NUM of posts.
 * Gets the first post that was posted by someone other than yourself (i.e., not with your principal ID).
   (Since you cannot Yeah your own post.)
 * Yeahs that post, or removes the Yeah from that post.
 
 To remove a Yeah, enable the UploadEmpathyToPostData_RemoveSample function (which is commented out in the demo).
 
 *---------------------------------------------------------------------------
*/
#include <nn.h>
#include <nn/olv.h>
#include <nn/act/act_Api.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for the post data.
    const u32 DOWNLOAD_MAX_NUM = 10;
    nn::olv::DownloadedPostData s_postDataList[DOWNLOAD_MAX_NUM];
    nn::olv::DownloadedTopicData s_topicData;

    // Your own principal ID.
    nn::friends::PrincipalId s_myPrincipalId;
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "UploadEmpathyToPostData demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

bool GetPostIdFromDownloadData(char *postIdBuffer, const u32 postIdBufferSize)
{
    nn::olv::Result result;

    // Download the post data, and get the ID of the post to Yeah.
    nn::olv::DownloadPostDataListParam downloadParam;
    
    downloadParam.SetPostDataMaxNum(DOWNLOAD_MAX_NUM);
 
    // Set the flag
    u32 flags = 0;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_FILTER_BY_BODY_TEXT;
    flags |= nn::olv::DownloadPostDataListParam::FLAG_WITH_EMPATHY_ADDED;
    downloadParam.SetFlags(flags);
 
    u32 downloadListSize = 0;
       
    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadPostDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    result = nn::olv::DownloadPostDataList(
            &s_topicData, s_postDataList,
            &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::DownloadPostDataList failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }

    // Search through posts -- excluding your own posts -- and copy the post IDs.
    for (s32 i = 0; i < downloadListSize; ++i)
    {
        if (s_postDataList[i].GetUserPid() != s_myPrincipalId)
        {
            char postIdTmpBuff[nn::olv::POST_ID_BUFF_LENGTH];
            s_postDataList[i].GetPostId(postIdTmpBuff, nn::olv::POST_ID_BUFF_LENGTH);
            strncpy(postIdBuffer, postIdTmpBuff, postIdBufferSize);
            return true;
        }
    }

    // Cannot make a Yeah because could not find any post data other than your own.
    NN_LOG("ERROR: GetPostIdFromDownloadData download success. but target post not found.\n");

    return false;
}

// Sample functions for making a Yeah to post data.
void UploadEmpathyToPostData_Sample(const char* postId)
{
    nn::olv::Result result;
    
    // Set the post ID of the post data to Yeah.
    nn::olv::UploadEmpathyToPostDataParam uploadParam;
    uploadParam.SetPostId(postId);
     
    // Get and set the work size.
    u32 workSize = uploadParam.GetWorkSize();
    NN_LOG("nn::olv::UploadEmpathyToPostData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    uploadParam.SetWork(workProcessMemory.buf, workSize);

    // Yeah the post data.
    result = nn::olv::UploadEmpathyToPostData(&uploadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadEmpathyToPostData failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
     
    NN_LOG("nn::olv::UploadEmpathyToPostData success.\n");
    NN_LOG("PostId: %s <--- Empathy is set.\n\n", postId);
}

// Sample functions for removing a Yeah from a post data.
void UploadEmpathyToPostData_RemoveSample(const char* postId)
{
    nn::olv::Result result;
    
    // Set the post ID of the post data to remove the Yeah from.
    nn::olv::UploadEmpathyToPostDataParam uploadParam;
    uploadParam.SetPostId(postId);
    
    // Set the flag representing removal of a Yeah.
    uploadParam.SetFlags(nn::olv::UploadEmpathyToPostDataParam::FLAG_REMOVE);
     
    // Get and set the work size.
    u32 workSize = uploadParam.GetWorkSize();
    NN_LOG("nn::olv::UploadEmpathyToPostData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return;
    }
    uploadParam.SetWork(workProcessMemory.buf, workSize);

    // Remove the Yeah from the post data.
    result = nn::olv::UploadEmpathyToPostData(&uploadParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadEmpathyToPostData failed.\n");
        OLVDemo_ErrorHandling(result);
        return;
    }
     
    NN_LOG("nn::olv::UploadEmpathyToPostData success.\n");
    NN_LOG("PostId: %s Empathy is removed.\n\n", postId);
}

void DemoMain()
{
    // From the post data, get the IDs of the post data that have been Yeahed.
    // Download and get the IDs for posts that are not yours.
    char postId[nn::olv::POST_ID_BUFF_LENGTH];
    if (!GetPostIdFromDownloadData(postId, nn::olv::POST_ID_BUFF_LENGTH))
    {
        NN_LOG("ERROR: GetPostIdFromDownloadData failed.\n");
        return;
    }
    
    // Yeah the post data.
    UploadEmpathyToPostData_Sample(postId);

    // To remove a Yeah from a post data, enable the following code.
    //UploadEmpathyToPostData_RemoveSample(postId);
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Get your own principal ID.
    {
        s_myPrincipalId = nn::act::GetPrincipalId();
        if (s_myPrincipalId == NN_ACT_INVALID_PRINCIPAL_ID)
        {
            NN_LOG("this account is not network account.\n");
            return false;
        }
        NN_LOG("my PrincipalId is %u\n", s_myPrincipalId);
    }

    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
