﻿/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     olv_UploadFavoriteToCommunityData_demo.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 This is a demo of registering communities to Favorites.
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/olv.h>
#include "demo.h"
#include "../common/common.h"

namespace {

    bool s_isEndDemo = false;

    // Working memory for initializing the OLV library.
    OLVDemo_MemoryManager s_olvInitializeMemory;

    // Parameters for initializing the OLV library.
    nn::olv::InitializedData s_initializedData;
    nn::olv::InitializeParam s_initializeParam;

    // Parameters for connecting the OLV library.
    nn::olv::ConnectedData s_connectedData;
    nn::olv::ConnectParam s_connectParam;

    // Container for downloaded data.
    const u32 DOWNLOAD_MAX_NUM = 1;
    nn::olv::DownloadedCommunityData s_communityDataList[DOWNLOAD_MAX_NUM];
}


namespace sample { namespace olv {

void ProceedDisplay(demo::RenderSystemDrawing &renderSystem)
{
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SetColor(1.0f, 1.0f, 1.0f);

    renderSystem.DrawText(0, LINE_HEIGHT, "UploadFavoriteToCommunityData demo");

    renderSystem.SwapBuffers();
}

void ProceedHid()
{
    static nn::hid::PadReader padReader;
    static nn::hid::PadStatus padStatus;

    if (!padReader.ReadLatest(&padStatus))
    {
        nn::dbg::detail::TPrintf("Getting pad status failed.\n");
    }
}

// Download the community data.
bool DownloadCommunityDataSample()
{
    nn::olv::DownloadCommunityDataListParam downloadParam;
    nn::olv::Result result;

    downloadParam.SetCommunityDataMaxNum(DOWNLOAD_MAX_NUM);

    u32 flags = 0;
    flags |= nn::olv::DownloadCommunityDataListParam::FLAG_FILTER_BY_OFFICIAL; // Get only official communities.
    downloadParam.SetFlags(flags);

    // Get and set the work size.
    u32 workSize = downloadParam.GetWorkSize();
    NN_LOG("nn::olv::DownloadCommunityDataList - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    downloadParam.SetWork(workProcessMemory.buf, workSize);

    // Download the community data.
    u32 downloadListSize = 0;
    result = nn::olv::DownloadCommunityDataList(
            s_communityDataList, &downloadListSize, &downloadParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("ERROR: nn::olv::DownloadCommunityDataList failed.\n");
        return false;
    }

    if (downloadListSize == 0)
    {
        NN_LOG("downloadListSize is 0.\n");
        return false;
    }

    return true;
}

// Register as favorite/Remove as favorite.
bool FavoriteToCommunityDataSample()
{
    // Parameter when registering/unregistering a community as a Favorite.
    nn::olv::UploadFavoriteToCommunityDataParam favoriteParam;
    
    nn::olv::Result result;
    
    // Set the community ID or community code for the community you want to register as a favorite.
    // Use either of the following methods.
    {    
        // Register as a favorite by setting the community code.
        const u32 communityCodeBuffSize = nn::olv::COMMUNITY_CODE_BUFF_LENGTH;
        char communityCode[communityCodeBuffSize];
        s_communityDataList[0].GetCommunityCode(communityCode, communityCodeBuffSize);
        // Set community code.
        favoriteParam.SetCommunityCode(communityCode);

        // Register as a favorite by setting the community ID.
        //u32 communityId = s_communityDataList[0].GetCommunityId();
        // Set the community ID.
        //favoriteParam.SetCommunityId(communityId);
    }
    
    // Set this flag if you want to remove a favorite.
    // This setting is not necessary when you are registering a favorite.
    //u32 flags = 0;
    //flags |= nn::olv::UploadFavoriteToCommunityDataParam::FLAG_REMOVE;
    //favoriteParam.SetFlags(flags);
  
    // Get and set the work size.
    u32 workSize = favoriteParam.GetWorkSize();
    NN_LOG("nn::olv::UploadFavoriteToCommunityData - work size:%d\n", workSize);
    OLVDemo_MemoryManager workProcessMemory;
    if ((workProcessMemory.Allocate(workSize)) == NULL)
    {
        NN_LOG("heap allocate failed. size:%d\n", workSize);
        return false;
    }
    favoriteParam.SetWork(workProcessMemory.buf, workSize);

    // Register community as a Favorite.
    result = nn::olv::UploadFavoriteToCommunityData(&favoriteParam);
    if (result.IsFailure())
    {
        NN_LOG("ERROR: nn::olv::UploadFavoriteToCommunityData failed.\n");
        OLVDemo_ErrorHandling(result);
        return false;
    }
     
    NN_LOG("nn::olv::UploadFavoriteToCommunityData success.\n");

    return true;
}

void DemoMain()
{
    if (!DownloadCommunityDataSample())
    {
        return;
    }

    FavoriteToCommunityDataSample();
}

bool MainLoop(demo::RenderSystemDrawing &renderSystem)
{
    ProceedHid();
    ProceedDisplay(renderSystem);

    if (nn::olv::IsConnected())
    {
        s_isEndDemo = true;
        DemoMain();
    }

    nngxWaitVSync(NN_GX_DISPLAY_BOTH);

    return s_isEndDemo;
}

bool Initialize(demo::RenderSystemDrawing&)
{
    // Set the parameter for initializing the OLV library.
    s_initializedData.Reset();
    s_initializeParam.Reset();

    // Get the needed size of working memory for initialization and allocate that memory.
    u32 workSize = s_initializeParam.GetWorkSize();
    NN_LOG("nn::olv::Initialize - work size:%d\n", workSize);
    if (!s_olvInitializeMemory.Allocate(workSize))
    {
        NN_LOG("failed allocate memory for olv initialize.\n");
        return false;
    }
    // Set the working memory for initialization.
    s_initializeParam.SetWork(s_olvInitializeMemory.buf, workSize);

    // Initialize the OLV library.
    nn::olv::Result result = nn::olv::Initialize(&s_initializedData, &s_initializeParam);
    if (result.IsFailure())
    {   // Initialization failed.
        s_olvInitializeMemory.Free();
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Initialize failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Initialize success.\n");

    return true;
}

bool Connect(demo::RenderSystemDrawing&)
{
    s_connectedData.Reset();
    s_connectParam.Reset();

    // Sets the service token.
    s_connectParam.SetServiceToken(OLVDemo_GetServiceToken());

    // Connect.
    nn::olv::Result result = nn::olv::Connect(&s_connectedData, &s_connectParam);
    if (result.IsFailure())
    {
        OLVDemo_ErrorHandling(result);
        NN_LOG("nn::olv::Connect failed.\n");
        return false;
    }

    NN_LOG("nn::olv::Connect success.\n");

    return true;
}

void Finalize()
{
    nn::olv::Finalize();
    s_olvInitializeMemory.Free();
    NN_LOG("nn::olv::Finalize success.\n");
}


}}
