/*---------------------------------------------------------------------------*
  Project:  OLV
  File:     common.cpp

  Copyright (C) 2009-2013 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#include <nn/olv.h>
#include <nn.h>
#include <nn/types.h>
#include <nn/dbg.h>
#include <nn/nstd.h>
#include <nn/gx.h>
#include <nn/fs.h>
#include <nn/act/act_Api.h>
#include <nn/act/act_ApiApplet.h>
#include <nn/applet.h>
#include <stdio.h>
#include <stdlib.h>
#include "demo.h"
#include "../common/common.h"

namespace sample { namespace olv {

// The following are the functions implemented in the various demos.
bool MainLoop(demo::RenderSystemDrawing &renderSystem);
bool Initialize(demo::RenderSystemDrawing &renderSystem);
bool Connect(demo::RenderSystemDrawing &renderSystem);
void Finalize();

}} // namespace sample::olv

// Internal state definitions.
enum
{
    OLV_DEMO_PROCESS_INIT = 0,                  // Initialize.
    OLV_DEMO_PROCESS_GET_SERVICETOKEN,          // Get service token.
    OLV_DEMO_PROCESS_OLV_CONNECT,               // Connect OLV library.
    OLV_DEMO_PROCESS_DEMO                       // Execution state of each demo.
};

namespace
{
    // Internal state.
    u32 s_processState = OLV_DEMO_PROCESS_INIT;   

    nn::os::LightEvent s_awakeEvent;
    volatile bool s_isEventInitialized = false;
    nn::os::CriticalSection s_csForEvent = nn::WithInitialize();
    nn::os::CriticalSection s_csForAwake = nn::WithInitialize();

    // Error Code
    u32 s_errorCode = 0;

    // Flag for whether to start the Error/EULA display applet.
    bool s_isToGoErrEula = false;

    // Flag indicating whether the OLV library is available.
    bool s_olvDisable = false;

    // Flag for whether to get the service token and to call the Connect function.
    bool s_isNeedToConnect = true;

    // Heap for the demo.
    nn::fnd::ExpHeap s_appHeap;

    // Service token.
    char s_serviceToken[NN_ACT_INDEPENDENT_SERVICE_TOKEN_SIZE];

    // Working memory for nn::fs.
    void* s_fsWorkingMemory = NULL;
    const char* s_fsArchiveName = "rom";
}

using namespace sample::olv;

AppletQueryReply myReceiveSleepQueryNotification( uptr arg )
{
    nn::os::CriticalSection::ScopedLock lock(s_csForAwake);
    nn::os::CriticalSection::ScopedLock lock2(s_csForEvent);
    NN_UNUSED_VAR( arg );
    NN_LOG("*** sleepQuery (isActive=%d) (enable=%d)\n", nn::applet::IsActive(), nn::applet::IsEnableSleep());

    if ( nn::applet::IsActive() )
    {
        if ( s_isEventInitialized )
        {
            s_awakeEvent.ClearSignal();
            return nn::applet::REPLY_LATER;
        }
        else
        {
            return nn::applet::REPLY_REJECT;
        }
    }
    else
    {
        return nn::applet::REPLY_ACCEPT;
    }
}

void myReceiveAwakeNotification( uptr arg )
{
    NN_UNUSED_VAR( arg );
    NN_LOG("*** awake \n");

    nn::os::CriticalSection::ScopedLock lock(s_csForEvent);
    if ( s_isEventInitialized )
    {
        s_awakeEvent.Signal();
    }
}

extern "C" void nninitStartUp(void)
{
    nn::os::SetDeviceMemorySize( 16 * 1024 * 1024 );
    nn::os::SetHeapSize( 16 * 1024 * 1024 );
}

void FsUnmount()
{
    nn::Result result = nn::fs::Unmount(s_fsArchiveName);
    if (result.IsFailure())
    {
        if(nn::fs::ResultNotFound::Includes(result))
        {
            NN_LOG("%s is not found!\n", s_fsArchiveName);
        }
        else
        {
            NN_LOG("Failed to unmount %s!\n", s_fsArchiveName);
        }
    }
    else
    {
        if (s_fsWorkingMemory)
        {
            OLVDemo_FreeMemory(s_fsWorkingMemory);
            s_fsWorkingMemory = NULL;
        }
    }
}

void FsMountRom()
{
    const size_t    maxFiles        = 32;
    const size_t    maxDirectories  = 8;
    const bool      useCache        = true;

    s32 workingMemorySize = nn::fs::GetRomRequiredMemorySize(maxFiles, maxDirectories, useCache);

    s_fsWorkingMemory = OLVDemo_AllocateMemory(workingMemorySize);
    if(!s_fsWorkingMemory)
    {
        NN_ASSERTMSG(0, "fs: Cannot allocate working memory!\n");
    }

    nn::fs::MountRom(s_fsArchiveName, maxFiles, maxDirectories, s_fsWorkingMemory, workingMemorySize, useCache);
}

void nnMain()
{
    nn::Result result;

    // Initialization of necessary related libraries.
    nn::fs::Initialize();
    nn::cfg::Initialize();
    result = nn::ac::Initialize();
    if (result.IsFailure())
    {
        u32 error;
        nn::ac::GetLastErrorCode(&error);
        // In the full retail version, conduct the appropriate error handling with the application, and do not use features that require Internet communication in the OLV library.
        NN_ASSERTMSG(0, "nn::ac::Initialize failed. errorCode:%d\n", error);
    }
    result = nn::act::Initialize();
    if (result.IsFailure())
    {
        // In the full retail version, conduct the appropriate error handling with the application, and do not use the OLV library.
        NN_ASSERTMSG(0, "nn::act::Initialize failed. errorCode:%d\n", nn::act::GetErrorCode(result));
    }
    result = nn::http::Initialize();
    if (result.IsFailure())
    {
        // In the full retail version, conduct the appropriate error handling with the application, and do not use features that require Internet communication in the OLV library.
        NN_ASSERTMSG(0, "nn::http::Initialize failed. description:%d\n", result.GetDescription());
    }
    result = nn::hid::Initialize();
    if (result.IsFailure())
    {
        // In the full retail version, conduct the appropriate error handling with the application.
        NN_ASSERTMSG(0, "nn::hid::Initialize failed. description:%d\n", result.GetDescription());
    }


    const f32 LINE_WIDTH    =  1.f;

    {
        nn::os::CriticalSection::ScopedLock lock(s_csForEvent);
        s_awakeEvent.Initialize(true);
        s_isEventInitialized = true;
    }

    // Initialize internal flag.
    s_olvDisable = false;

    // applet declarations
    nn::applet::SetSleepQueryCallback( myReceiveSleepQueryNotification, 0 );
    nn::applet::SetAwakeCallback( myReceiveAwakeNotification, 0 );
    nn::applet::Enable();

    // Initialize for rendering.
    nn::fnd::ExpHeap heap;
    heap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize() );

    uptr heapForGx = reinterpret_cast<uptr>(heap.Allocate(0x800000));
    demo::RenderSystemDrawing renderSystem;
    renderSystem.Initialize(heapForGx, 0x800000);
    renderSystem.SetClearColor(NN_GX_DISPLAY_BOTH, 0.f, 0.f, 0.f, 0.f);
    renderSystem.SetFontSize(FONT_SIZE);
    renderSystem.SetLineWidth(LINE_WIDTH);
    renderSystem.SetRenderTarget(NN_GX_DISPLAY0);
    renderSystem.Clear();
    renderSystem.SwapBuffers();
    renderSystem.SetRenderTarget(NN_GX_DISPLAY1);
    renderSystem.Clear();
    renderSystem.SwapBuffers();

    // Initialize the heap for the demo.
    s_appHeap.Initialize(nn::os::GetHeapAddress(), nn::os::GetHeapSize());

    // Mount so the data for the demo can be used.
    FsMountRom();

    nn::applet::EnableSleep();

    bool isEndDemo = false;

    while (1)
    {
        s_isToGoErrEula = false;

        while (1)
        {
            // The processes of the demo.
            if (MainLoop(renderSystem))
            {
                isEndDemo = true;
                break;
            }

            // Sleep
            if ( nn::applet::IsExpectedToReplySleepQuery() )
            {
                nn::os::CriticalSection::ScopedLock lock(s_csForAwake);

                nn::applet::ReplySleepQuery( nn::applet::REPLY_ACCEPT );

                s_awakeEvent.Wait();
                nngxStartLcdDisplay();
            }

            // Request power button process.
            if ( nn::applet::IsExpectedToProcessPowerButton() )
            {
                nn::applet::DisableSleep();
                nn::applet::ProcessPowerButton();
                break;
            }

            switch (s_processState)
            {
                case OLV_DEMO_PROCESS_INIT:
                    {
                        if (Initialize(renderSystem))
                        {   // Go to the next sequence if initialization was successful.
                            if (s_isNeedToConnect)
                            {
                                s_processState = OLV_DEMO_PROCESS_GET_SERVICETOKEN;
                            }
                            else
                            {
                                s_processState = OLV_DEMO_PROCESS_DEMO;
                            }
                        }
                        else
                        {   // End the demo if it failed.
                            isEndDemo = true;
                        }
                    }
                    break;

                case OLV_DEMO_PROCESS_GET_SERVICETOKEN:
                    {

                        if (!nn::ac::IsConnected())
                        {
                            nn::ac::Config config;

                            result = nn::ac::CreateDefaultConfig( &config );
                            NN_UTIL_PANIC_IF_FAILED(result);

                            NN_LOG("ac::Connect...\n");

                            // The ac connection process.
                            result = nn::ac::Connect( config );
                            if (result.IsSuccess())
                            {
                                NN_LOG("ac::Connect success\n");
                            }
                            else
                            {
                                NN_LOG("ac::Connect failed. %08X\n", result.GetPrintableBits());
                                nn::ac::GetLastErrorCode(&s_errorCode);
                                NN_LOG("errorCode:%d\n", s_errorCode);
                                break;
                            }
                        }

                        nn::act::applet::Preload();
                        nn::act::applet::WaitForPreload();
                        result = nn::act::applet::AcquireOlvServiceToken(s_serviceToken);
                        if (result.IsSuccess())
                        {
                            NN_LOG("AcquireOlvServiceToken success\n");
                            // Transition to connected state.
                            s_processState = OLV_DEMO_PROCESS_OLV_CONNECT;
                        }
                        else
                        {
                            NN_LOG("AcquireOlvServiceToken failed. %08X\n", result.GetPrintableBits());
                            s_errorCode = nn::act::GetErrorCode(result);
                        }

                        // Recover the GPU register settings
                        nngxUpdateState(NN_GX_STATE_ALL);
                        nngxValidateState(NN_GX_STATE_ALL,GL_TRUE);
                    }
                    break;

                case OLV_DEMO_PROCESS_OLV_CONNECT:
                    {   // Separate demo connection process.
                        if (Connect(renderSystem))
                        {
                            // Transition to running the demo.
                            s_processState = OLV_DEMO_PROCESS_DEMO;
                        }
                        else
                        {   // End the demo if it failed.
                            isEndDemo = true;
                        }
                    }
                    break;
            }

            if (s_errorCode != 0)
            {
                // Show errors with the Error/EULA display applet.
                s_isToGoErrEula = true;
                break;
            }
        }

        if (isEndDemo && s_errorCode == 0)
        {   // The function completed successfully.
            break;
        }

        if (s_isToGoErrEula)
        {
            static nn::erreula::Parameter errEulaParam;
            nn::applet::AppletWakeupState wState;
            NN_UNUSED_VAR(wState);

            nn::erreula::InitializeConfig( &errEulaParam.config );
            errEulaParam.config.errorCode = s_errorCode;

            // Show errors with the Error/EULA display applet.
            nn::erreula::StartErrEulaApplet(&wState, &errEulaParam);

            // This demo ends after the Error/EULA display applet displays.
            isEndDemo = true;

            s_errorCode = 0;
        }
        else
        {
            nn::applet::WaitForStarting();
        }

        if (isEndDemo || nn::applet::CTR::IsExpectedToCloseApplication())
        {
            break;
        }
    }

    Finalize();

    FsUnmount();

    s_appHeap.Finalize();
    heap.Free(reinterpret_cast<void*>(heapForGx));
    heap.Finalize();
}

void OLVDemo_SetErrorCode(u32 errorCode)
{
    s_errorCode = errorCode;
}

void OLVDemo_ErrorHandling(const nn::olv::Result& result)
{
    if (result <= nn::olv::ResultLibraryPermanentError() || 
        result <= nn::olv::ResultServerPermanentError())
    {
        // Show errors with the Error/EULA display applet.
        s_errorCode = nn::olv::GetErrorCode(result);
        s_isToGoErrEula = true;

        // Set the OLV library to not use what comes after this point.
        s_olvDisable = true;
    }
    else if (result <= nn::olv::ResultLibraryTemporaryError() || 
        result <= nn::olv::ResultNetworkError() || 
        result <= nn::olv::ResultServerTemporaryError())
    {
        // Show errors with the Error/EULA display applet.
        s_errorCode = nn::olv::GetErrorCode(result);
        s_isToGoErrEula = true;

        // If possible, make it so the user input can be tried again.
    }
    else if (result <= nn::olv::ResultFatalError() || 
        result <= nn::olv::ResultUsageError())
    {
        // Show errors with the Error/EULA display applet.
        s_errorCode = nn::olv::GetErrorCode(result);
        s_isToGoErrEula = true;

        NN_LOG("OLV Error occured. errorCode is 0%d\n", s_errorCode);

        // Make this so it is removed in debugging and ignored in the product version.
        NN_ASSERTMSG(0, "Error: %u", nn::olv::GetErrorCode(result));
    }
    else
    {
        // This error does not display an error, so ignore it.
    }

    NN_LOG("OLV Error occured. errorCode is 0%d\n", s_errorCode);

    if (s_olvDisable)
    {   // The OLV library cannot be used beyond here.
        NN_LOG("can not use the OLV library...\n");
    }
}

bool OLVDemo_IsOlvEnabled()
{
    return !s_olvDisable;
}

u8* OLVDemo_AllocateMemory(u32 size)
{
    return static_cast<u8*>(s_appHeap.Allocate(size));
}

void OLVDemo_FreeMemory(void* ptr)
{
    s_appHeap.Free(ptr);
}

bool OLVDemo_ReadFile(const char* fileName, void*& outBuffer, size_t& readSize)
{
    const u32 FULL_PATH_FILE_NAME_SIZE = 256;
    char fullPathFileName[FULL_PATH_FILE_NAME_SIZE] = "";
    snprintf(fullPathFileName, FULL_PATH_FILE_NAME_SIZE, "%s%s%s", s_fsArchiveName, ":/", fileName);

    nn::fs::FileInputStream f(fullPathFileName);

    readSize = f.GetSize();
    NN_LOG("readSize:%d\n", readSize);
    outBuffer = OLVDemo_AllocateMemory(readSize);
    if(!outBuffer)
    {
        NN_LOG("OLVDemo_ReadFile: Cannot allocate memory. size=%d\n", readSize);
        return false;
    }

    f.Read(outBuffer, readSize);

    return true;
}

void OLVDemo_PrintWideStr(const wchar_t* str, u32 num)
{
    int i = 0;
    if (num == 0)
    {
        while (str[i] != 0x0000)
        {
            NN_LOG("0x%x,", str[i]);
            i++;
        }
    }
    else
    {
        for (i=0; i<num; ++i)
        {
            NN_LOG("0x%x,", str[i]);
        }
    }

    NN_LOG("\n");
}

void OLVDemo_PrintBinary(u8* data, u32 num)
{
    for (int i=0; i<num; ++i)
    {
        NN_LOG("0x%x,", data[i]);
    }
    NN_LOG("\n");
}

void OLVDemo_SetIsConnect(bool isConnect)
{
    s_isNeedToConnect = isConnect;
}

const char* OLVDemo_GetServiceToken()
{
    return s_serviceToken;
}

void OLVDemo_PrintDate(nn::fnd::DateTime date)
{
    NN_LOG("Date: %d/%d/%d %02d:%02d:%02d\n", 
        date.GetYear(), date.GetMonth(), date.GetDay(), date.GetHour(), date.GetMinute(), date.GetSecond());
}




